import glpk
import random

def pair2index(i, j):
    """Convenience function to get an item pair's variable index."""
    i, j = min(i, j), max(i, j)
    return (j * (j-1)) // 2 + i

def pretty_print(matrix):
    """Outputs a 'matrix' in an easily readable format."""
    for row in matrix[1:]:
        print(' '.join('%.3f'%value for value in row))

def gen_random(items, rgen = random):
    """Generate a random matrix."""
    return [[rgen.uniform(-1,1) for i in range(j)] for j in range(items)]

def correlation_clustering(similarity):
    """Linear program relaxation of correlation clustering."""
    lp = glpk.LPX()                          # Define the linear program.
    items = len(similarity)                  # Get the number of items.
    lp.obj.maximize = True                   # Set as maximization.
    lp.cols.add((items * (items-1)) // 2)    # Each item pair has a var.
    for j in range(items):
        for i in range(j):
            index = pair2index(i, j)         # Get the index for this pair.
            lp.cols[index].bounds = 0, 1     # Each variable in range 0 to 1.
            lp.obj[index] = similarity[j][i] # If 1, this much added to obj.
    for k in range(items):                  # For all triples of items, we
        for j in range(k):                  # want to add in constraints to
            jk = pair2index(j,k)             # enforce respect for the
            for i in range(j):              # triangle inequality.
                ij, ik = pair2index(i,j), pair2index(i,k)
                # We want Xij >= Xik + Xjk - 1.
                # That is, we want 1 >= Xik + Xjk - Xij.
                # Add constraints to enforce this!
                lp.rows.add(3)
                lp.rows[-3].matrix = (ij, 1), (ik, 1), (jk,-1)
                lp.rows[-2].matrix = (ij, 1), (ik,-1), (jk, 1)
                lp.rows[-1].matrix = (ij,-1), (ik, 1), (jk, 1)
    for row in lp.rows:                      # For each row.
        row.bounds = None, 1                 # Each row is <= 1.
    glpk.env.term_on = False                 # Make it shut up.
    lp.simplex()                             # Run the optimization.
    labels = [[lp.cols[pair2index(i,j)].value for i in range(j)]
              for j in range(items)]
    return lp.obj.value, labels

def halfint_correlation_clustering(similarity):
    """Wacky MIP-based half-int constrained correlation clustering."""
    lp = glpk.LPX()                          # Define the linear program.
    items = len(similarity)                  # Get the number of items.
    lp.obj.maximize = True                   # Set as maximization.
    lp.cols.add((items * (items-1)) // 2)    # Each item pair has a var.
    for j in range(items):
        for i in range(j):
            index = pair2index(i, j)         # Get the index for this pair.
            lp.cols[index].bounds = 0, 2     # Each variable in range 0 to 1.
            lp.cols[index].kind = int        # This should be integral.
            lp.obj[index]=similarity[j][i]/2 # If 1, this much added to obj.
    for k in range(items):                  # For all triples of items, we
        for j in range(k):                  # want to add in constraints to
            jk = pair2index(j,k)             # enforce respect for the
            for i in range(j):              # triangle inequality.
                ij, ik = pair2index(i,j), pair2index(i,k)
                # We want Xij >= Xik + Xjk - 1.
                # That is, we want 1 >= Xik + Xjk - Xij.
                # Add constraints to enforce this!
                lp.rows.add(3)
                lp.rows[-3].matrix = (ij, 1), (ik, 1), (jk,-1)
                lp.rows[-2].matrix = (ij, 1), (ik,-1), (jk, 1)
                lp.rows[-1].matrix = (ij,-1), (ik, 1), (jk, 1)
    for row in lp.rows:                      # For each row.
        row.bounds = None, 2                 # Each row is <= 1.
    glpk.env.term_on = False                 # Make it shut up.
    lp.simplex()                             # Find the basic solution.
    lp.integer()                             # Find the integer solution.
    labels = [[lp.cols[pair2index(i,j)].value/2 for i in range(j)]
              for j in range(items)]
    return lp.obj.value, labels

# Run the random test.
for i in range(1000):
    if i%100 == 0: print('At seed', i, '...')
    sim = gen_random(7, random.Random(i))
    objval, labels = correlation_clustering(sim)
    # How far off being half integral are we?
    off_half_integer = sum(abs(2*v - round(2*v)) for r in labels for v in r)
    if off_half_integer > .1:
        # We have found a significantly non-half integral solution!
        print('Found for seed', i)
        print('SIMILARITY MATRIX:')
        pretty_print(sim)
        print('LABELS FROM CORRELATION CLUSTERING (objective=%g):' % objval)
        pretty_print(labels)
        # Find the half-int solution, and report it for comparison.
        halfint_objval, halfint_labels = halfint_correlation_clustering(sim)
        print('LABELS FROM HALF-INT CORRELATION CLUSTERING (objective=%g):' % (
            halfint_objval))
        pretty_print(halfint_labels)
        # We have found what we are looking for, so, stop looking.
        break



import glpk


def tsp(edges):
    # Maps node to col indices of incident edges.
    node2colnums = {}
    for colnum, edge in enumerate(edges):
        n1, n2, cost = edge
        node2colnums.setdefault(n1, []).append(colnum)
        node2colnums.setdefault(n2, []).append(colnum)

    # empty LP instance
    lp = glpk.LPX()

    # stop annoying messages
    glpk.env.term_on = False

    # A struct var for each edge
    lp.cols.add(len(edges))
    # constraint for each node
    lp.rows.add(len(node2colnums)+1)

    # try to minimize the total costs
    lp.obj[:] = [e[-1] for e in edges]
    lp.obj.maximize = False

    # make all struct variables binary
    for col in lp.cols:
        col.kind = bool

    # for each node, select two edges, i.e.., an arrival and a departure
    for row, edge_column_nums in zip(lp.rows, node2colnums.values()):
        row.matrix = [(cn, 1.0) for cn in edge_column_nums]
        row.bounds = 2

    # we should select exactly (number of nodes) edges total
    lp.rows[-1].matrix = [1.0]*len(lp.cols)
    lp.rows[-1].bounds = len(node2colnums)

    assert lp.simplex() is None
    # if no relaxed sol., no exact sol.
    if lp.status != 'opt':
        return None

    # should not fail this way
    assert lp.integer() is None
    if lp.status != 'opt':
        # could not find integer solution
        return None

    # return the edges whose associated struct var has value 1
    return [edge for edge, col in zip(edges, lp.cols) if col.value > 0.99]



import glpk

def solve_sat(expression):
    if len(expression)==0: return [] # Trivial case.  Otherwise count vars.
    numvars = max([max([abs(v) for v in clause]) for clause in expression])
    lp = glpk.LPX()                  # Construct an empty linear program.
    glpk.env.term_on = False         # Stop the annoying output.
    lp.cols.add(2*numvars)           # As many columns as there are literals.
    for col in lp.cols:              # Literal must be between false and true.
        col.bounds = 0.0, 1.0
    def lit2col(lit):                # Function to compute column index.
        return [2*(-lit)-1,2*lit-2][lit>0]
    for i in range(1, numvars+1):   # Ensure "oppositeness" of literals.
        lp.rows.add(1)
        lp.rows[-1].matrix = [(lit2col(i), 1.0), (lit2col(-i), 1.0)]
        lp.rows[-1].bounds = 1.0     # Must sum to exactly 1.
    for clause in expression:        # Ensure "trueness" of each clause.
        lp.rows.add(1)
        lp.rows[-1].matrix = [(lit2col(lit), 1.0) for lit in clause]
        lp.rows[-1].bounds = 1, None # At least one literal must be true.
    retval = lp.simplex()            # Try to solve the relaxed problem.
    assert retval is None            # Should not fail in this fashion.
    if lp.status!='opt': return None # If no relaxed solution, no exact sol.

    for col in lp.cols:
        col.kind = int
    retval = lp.integer()            # Try to solve this integer problem.
    assert retval is None            # Should not fail in this fashion.
    if lp.status != 'opt': return None
    return [col.value > 0.99 for col in lp.cols[::2]]

exp = [(-1, -3, -4), (2, 3, -4), (1, -2, 4), (1, 3, 4), (-1, 2, -3)]
print(solve_sat(exp))



"""Tests for reading and writing problems and solutions."""

import tempfile
import unittest

from glpk import LPX


class WriteTests(unittest.TestCase):
    """A simple suite of tests for this problem.

    max (x+y) subject to
    0.5*x + y <= 1
    0 <= x <= 1
    0 <= y <= 1

    This should have optimal solution x=1, y=0.5."""
    def setUp(self):
        # create a new temp file for reading/writing to
        self.f = tempfile.NamedTemporaryFile()
        lp = self.lp = LPX()
        lp.rows.add(1)
        lp.cols.add(2)
        lp.cols[0].name = 'x'
        lp.cols[1].name = 'y'
        for c in lp.cols:
            c.bounds = 0, 1
        lp.obj[:] = [1, 1]
        lp.obj.maximize = True
        lp.rows[0].matrix = [0.5, 1.0]
        lp.rows[0].bounds = None, 1

    def tearDown(self):
        # close (delete) the temp file
        self.f.close()

    def testMps(self):
        """Test reading/writing MPS format."""
        self.lp.write(mps=self.f.name)
        lp = LPX(mps=self.f.name)
        self.assertEqual(len(lp.rows), 1)
        self.assertEqual(len(lp.cols), 2)
        self.assertEqual(lp.cols[0].name, 'x')
        self.assertEqual(lp.cols[1].name, 'y')

        # try to read a non-existing file
        with self.assertRaises(RuntimeError) as cm:
            lp = LPX(mps='not a real file')
        self.assertIn('MPS reader failed', str(cm.exception))

        # try to write to a non-existing path
        with self.assertRaises(RuntimeError) as cm:
            lp.write(mps='not/a/real/file')
        self.assertIn(
            "writer for 'mps' failed to write to 'not/a/real/file'",
            str(cm.exception)
        )

    def testFreeMps(self):
        """Test reading/writing free MPS format."""
        self.lp.write(freemps=self.f.name)
        lp = LPX(freemps=self.f.name)
        self.assertEqual(len(lp.rows), 1)
        self.assertEqual(len(lp.cols), 2)
        self.assertEqual(lp.cols[0].name, 'x')
        self.assertEqual(lp.cols[1].name, 'y')

        # try to read a non-existing file
        with self.assertRaises(RuntimeError) as cm:
            lp = LPX(freemps='not a real file')
        self.assertIn('Free MPS reader failed', str(cm.exception))

        # try to write to a non-existing path
        with self.assertRaises(RuntimeError) as cm:
            lp.write(freemps='not/a/real/file')
        self.assertIn(
            "writer for 'freemps' failed to write to 'not/a/real/file'",
            str(cm.exception)
        )

    def testCpxLp(self):
        """Test reading/writing CPLEX LP format."""
        self.lp.write(cpxlp=self.f.name)
        lp = LPX(cpxlp=self.f.name)
        self.assertEqual(len(lp.rows), 1)
        self.assertEqual(len(lp.cols), 2)
        self.assertEqual(lp.cols[0].name, 'x')
        self.assertEqual(lp.cols[1].name, 'y')

        # try to read a non-existing file
        with self.assertRaises(RuntimeError) as cm:
            lp = LPX(cpxlp='not a real file')
        self.assertIn('CPLEX LP reader failed', str(cm.exception))

        # try to write to a non-existing path
        with self.assertRaises(RuntimeError) as cm:
            lp.write(cpxlp='not/a/real/file')
        self.assertIn(
            "writer for 'cpxlp' failed to write to 'not/a/real/file'",
            str(cm.exception)
        )

    def testGlp(self):
        """Test reading/writing GNU LP format."""
        self.lp.write(glp=self.f.name)
        lp = LPX(glp=self.f.name)
        self.assertEqual(len(lp.rows), 1)
        self.assertEqual(len(lp.cols), 2)
        self.assertEqual(lp.cols[0].name, 'x')
        self.assertEqual(lp.cols[1].name, 'y')

        # try to read a non-existing file
        with self.assertRaises(RuntimeError) as cm:
            lp = LPX(glp='not a real file')
        self.assertIn('GLPK LP/MIP reader failed', str(cm.exception))

        # try to write to a non-existing path
        with self.assertRaises(RuntimeError) as cm:
            lp.write(glp='not/a/real/file')
        self.assertIn(
            "writer for 'glp' failed to write to 'not/a/real/file'",
            str(cm.exception)
        )

    def testGmp(self):
        """Test reading/writing GNU MathProg format."""
        # there is no write keyword for 'gmp'; write the file manually
        self.f.write('''
        var x, >= 0, <= 1;
        var y, >= 0, <= 1;
        maximize
        value: x + y;
        subject to
        row: .5*x + y <= 1;
        end;
        '''.encode('utf-8'))
        self.f.flush()
        lp = LPX(gmp=self.f.name)
        # TODO: why is this two?
        self.assertEqual(len(lp.rows), 2)
        self.assertEqual(len(lp.cols), 2)
        self.assertEqual(lp.cols[0].name, 'x')
        self.assertEqual(lp.cols[1].name, 'y')

        # try to read a non-existing file
        with self.assertRaises(RuntimeError) as cm:
            lp = LPX(gmp='not a real file')
        self.assertIn('GMP model reader failed', str(cm.exception))

        # try to write to a non-existing path
        with self.assertRaises(ValueError) as cm:
            lp = LPX(gmp=())
        self.assertIn('model tuple must have 1<=length<=3', str(cm.exception))

        with self.assertRaises(TypeError) as cm:
            lp = LPX(gmp={})
        self.assertIn('model arg must be string or tuple', str(cm.exception))

    def testWriteSol(self):
        """Test writing basic solution."""
        self.lp.simplex()
        self.lp.write(sol=self.f.name)

        # try to write to a non-existing path
        with self.assertRaises(RuntimeError) as cm:
            self.lp.write(sol='not/a/real/file')
        self.assertIn(
            "writer for 'sol' failed to write to 'not/a/real/file'",
            str(cm.exception)
        )

    def testWriteSensBnds(self):
        """Test writing bounds sensitivity."""
        self.lp.simplex()
        self.lp.write(sens_bnds=self.f.name)

        # try to write to a non-existing path
        with self.assertRaises(RuntimeError) as cm:
            self.lp.write(sens_bnds='not/a/real/file')
        self.assertIn(
            "writer for 'sens_bnds' failed to write to 'not/a/real/file'",
            str(cm.exception)
        )

    def testWriteIps(self):
        """Test writing interior-point solution."""
        self.lp.interior()
        self.lp.write(ips=self.f.name)

        # try to write to a non-existing path
        with self.assertRaises(RuntimeError) as cm:
            self.lp.write(ips='not/a/real/file')
        self.assertIn(
            "writer for 'ips' failed to write to 'not/a/real/file'",
            str(cm.exception)
        )

    def testWriteMip(self):
        """Test writing MIP solution."""
        self.lp.simplex()
        self.lp.integer()
        self.lp.write(mip=self.f.name)

        # try to write to a non-existing path
        with self.assertRaises(RuntimeError) as cm:
            self.lp.write(mip='not/a/real/file')
        self.assertIn(
            "writer for 'mip' failed to write to 'not/a/real/file'",
            str(cm.exception)
        )



import glpk


def maxflow(capgraph, s, t):
    # map non-source/sink nodes to row num
    node2rnum = {}
    for nfrom, nto, cap in capgraph:
        if nfrom != s and nfrom != t:
            node2rnum.setdefault(nfrom, len(node2rnum))
        if nto != s and nto != t:
            node2rnum.setdefault(nto, len(node2rnum))

    # empty LP instance
    lp = glpk.LPX()

    # stop annoying messages
    glpk.env.term_on = False

    # as many columns cap-graph edges
    lp.cols.add(len(capgraph))
    # as many rows as non-source/sink nodes
    lp.rows.add(len(node2rnum))

    # net flow for non-source/sink is 0
    for row in lp.rows:
        row.bounds = 0

    # will hold constraint matrix entries
    mat = []

    for colnum, (nfrom, nto, cap) in enumerate(capgraph):
        # flow along edge bounded by capacity
        lp.cols[colnum].bounds = 0, cap

        if nfrom == s:
            # flow from source increases flow value
            lp.obj[colnum] = 1.0
        elif nto == s:
            # flow to source decreases flow value
            lp.obj[colnum] = -1.0

        if nfrom in node2rnum:
            # flow from node decreases its net flow
            mat.append((node2rnum[nfrom], colnum, -1.0))
        if nto in node2rnum:
            # flow to node increases its net flow
            mat.append((node2rnum[nto], colnum, 1.0))

    # want source s max flow maximized
    lp.obj.maximize = True
    # assign 0 net-flow constraint matrix
    lp.matrix = mat

    # this should work unless capgraph bad
    lp.simplex()

    # return edges with assigned flow
    return [
        (nfrom, nto, col.value)
        for col, (nfrom, nto, cap) in zip(lp.cols, capgraph)
    ]

capgraph = [
    ('s', 'a', 4), ('s', 'b', 1),
    ('a', 'b', 2.5), ('a', 't', 1), ('b', 't', 4)
]
print(maxflow(capgraph, 's', 't'))

[('s', 'a', 3.5), ('s', 'b', 1.0), ('a', 'b', 2.5),
 ('a', 't', 1.0), ('b', 't', 3.5)]



import sat
import sys
import threading

# At present, GLPK itself, and consequently PyGLPK, are not thread
# safe.  As a result, this "multithreaded" example at present does not
# run the threads in parallel during solving.  I am, however,
# retaining the example since I live in hope that

vars, clauses = 50, 200
num_threads = 8

def solve_problem(tid, num):
    exp = sat.generate_cnf(vars, clauses)
    assignment = sat.solve_sat(exp)
    if assignment: assert sat.verify(exp, assignment)
    sys.stdout.write((('+' if assignment else '-')+'(%d,%2d)')%(tid,num))
    sys.stdout.flush()

def thread_callable(tid):
    for i in range(50):
        solve_problem(tid, i)
    sys.stdout.write('F(%d)' % tid)
    sys.stdout.flush()

threads = []
for tid in range(num_threads):
    thread = threading.Thread(target=thread_callable, args=(tid,))
    threads.append(thread)
    thread.start()

for thread in threads:
    thread.join()



"""Tests for setting up the LP constraint matrix."""


import glpk
import random
import unittest


class MatrixCoefficientTestCase(unittest.TestCase):
    """Tests setting and reading the coefficient matrix."""
    def setUp(self):
        self.lp = glpk.LPX()
        self.lp.rows.add(3)
        self.lp.cols.add(4)
        for row in self.lp.rows:
            row.name = 'r%d' % row.index
        for col in self.lp.cols:
            col.name = 'c%d' % col.index
        self.matrix = [
            (0, 0, 2), (0, 1, 3), (0, 2, -2), (1, 1, 3.5), (1, 2, -5),
            (1, 3, 10), (2, 0, 7), (2, 1, 8), (2, 2, -3), (2, 3, 900)
        ]

    # Convenience methods for running comparison and extracting components.
    def rowVector(self, rnum, mat=None):
        if mat is None:
            mat = self.matrix
        return [(c, val) for r, c, val in mat if r == rnum]

    def colVector(self, cnum, mat=None):
        if mat is None:
            mat = self.matrix
        return [(r, val) for r, c, val in mat if c == cnum]

    def checkMatrix(self, mat=None):
        """Typical final equality check for our test functions."""
        # Compare matrix all at once, and also piecemeal by row and
        # column individually.
        if mat is None:
            mat = self.matrix
        self.assertEqual(self.lp.matrix, mat)
        for row in self.lp.rows:
            self.assertEqual(row.matrix, self.rowVector(row.index, mat))
        for col in self.lp.cols:
            self.assertEqual(col.matrix, self.colVector(col.index, mat))

    # This is the matrix used throughout this example:
    # [ 2,   3, -2,   0 ]
    # [ 0, 3.5, -5,  10 ]
    # [ 7,   8, -3, 900 ]

    def testInitialMatrix(self):
        """Test that the coefficient matrix is initially empty."""
        self.assertEqual(len(self.lp.matrix), 0)
        self.checkMatrix([])

    def testMatrixSetWithExplicitIndices(self):
        """Test setting the matrix with only explicit indices."""
        self.lp.matrix = self.matrix
        self.checkMatrix()

    def testMatrixSetWithExplicitStringIndices(self):
        """Test setting the matrix with explicit string indices."""
        matrix = [('r%d' % r, 'c%d' % c, v) for r, c, v in self.matrix]
        self.lp.matrix = matrix
        self.checkMatrix()

    def testMatrixSetWithExplicitIndicesOutOfOrder(self):
        """Test setting matrix with indices out of order."""
        rgen = random.Random(1)
        matrix = self.matrix[:]  # Shuffle only a copy.
        rgen.shuffle(matrix)
        self.lp.matrix = matrix
        self.checkMatrix()

    def testMatrixSetWithImplicitIndices(self):
        """Test setting the matrix with only implicit indices."""
        imatrix = [0.0 for r in range(len(self.lp.rows))
                   for c in range(len(self.lp.cols))]
        for r, c, v in self.matrix:
            imatrix[r*len(self.lp.cols) + c] = float(v)
        self.lp.matrix = imatrix
        self.checkMatrix()

    def testMatrixUnsetWithEmptyAssignment(self):
        """Test removing the matrix by assigning to []."""
        self.lp.matrix = self.matrix
        self.checkMatrix()
        self.lp.matrix = []
        self.checkMatrix([])

    def testMatrixUnsetWithDelete(self):
        """Test removing the matrix by a del upon it."""
        self.lp.matrix = self.matrix
        self.checkMatrix()
        del self.lp.matrix
        self.checkMatrix([])

    def testSetRowVectorsWithExplicitIndices(self):
        """Test setting the matrix row by row."""
        for row in self.lp.rows:
            row.matrix = self.rowVector(row.index)
        self.checkMatrix()

    def testSetRowVectorsWithExplicitStringIndices(self):
        """Test setting the matrix row by row with string column indices."""
        for row in self.lp.rows:
            row.matrix = [('c%d' % c, v) for c, v in self.rowVector(row.index)]
        self.checkMatrix()

    def testSetRowVectorsWithImplicitIndices(self):
        """Test setting the matrix row by row with no indices."""
        for row in self.lp.rows:
            vec = [0.0 for i in range(len(self.lp.cols))]
            for c, v in self.rowVector(row.index):
                vec[c] = v
            row.matrix = vec
        self.checkMatrix()

    def testSetColumnVectorsWithExplicitIndices(self):
        """Test setting the matrix column by column."""
        for col in self.lp.cols:
            col.matrix = self.colVector(col.index)
        self.checkMatrix()

    def testSetColumnVectorsWithExplicitStringIndices(self):
        """Test setting the matrix column by column with string row indices."""
        for col in self.lp.cols:
            col.matrix = [('r%d' % r, v) for r, v in self.colVector(col.index)]
        self.checkMatrix()

    def testSetColumnVectorsWithImplicitIndices(self):
        """Test setting the matrix column by column with no indices."""
        for col in self.lp.cols:
            vec = [0.0 for i in range(len(self.lp.rows))]
            for r, v in self.colVector(col.index):
                vec[r] = v
            col.matrix = vec
        self.checkMatrix()

    def testUnsetRowVectors(self):
        """Test unsetting rows with empty assignments and deletion."""
        mat = self.matrix
        # Initial assignment OK.
        self.lp.matrix = mat
        self.checkMatrix()
        # Try unsetting a row by assigning its matrix to [].
        self.lp.rows[1].matrix = []
        mat = [(r, c, v) for r, c, v in mat if r != 1]
        self.checkMatrix(mat)
        # Try unsetting a row by deleting the matrix.
        del self.lp.rows[2].matrix
        mat = [(r, c, v) for r, c, v in mat if r != 2]
        self.checkMatrix(mat)
        # Try unsetting a row with assignment to 0.
        self.lp.rows[0].matrix = [0, 0, 0, 0]
        mat = [(r, c, v) for r, c, v in mat if r != 0]
        self.checkMatrix(mat)

    def testUnsetColumnVectors(self):
        """Test unsetting rows with empty assignments and deletion."""
        mat = self.matrix
        # Initial assignment OK.
        self.lp.matrix = mat
        self.checkMatrix()
        # Try unsetting a row by assigning its matrix to [].
        self.lp.cols[1].matrix = []
        mat = [(r, c, v) for r, c, v in mat if c != 1]
        self.checkMatrix(mat)
        # Try unsetting a row by deleting the matrix.
        del self.lp.cols[2].matrix
        mat = [(r, c, v) for r, c, v in mat if c != 2]
        self.checkMatrix(mat)
        # Try unsetting a column with assignment to 0.
        self.lp.cols[0].matrix = [0, 0, 0]
        mat = [(r, c, v) for r, c, v in mat if c != 0]
        self.checkMatrix(mat)

    def testResetMatrix(self):
        """Tests setting the matrix, then setting it to something new."""
        mat = self.matrix
        # Initial assignment OK.
        self.lp.matrix = mat
        self.checkMatrix()
        # Shift the rows down one.
        mat = [((r+1) % len(self.lp.rows), c, v) for r, c, v in mat]
        self.lp.matrix = mat
        mat.sort()
        self.checkMatrix(mat)

    def testMatrixWithVectorDeletion(self):
        """Tests that the matrix is sane when rows/columns are deleted."""
        mat = self.matrix
        # Initial assignment OK.
        self.lp.matrix = mat
        self.checkMatrix()
        # Delete some row.
        del self.lp.rows[1]
        mat = [(r, c, v) for r, c, v in mat if r < 1] + [
            (r-1, c, v) for r, c, v in mat if r > 1]
        mat.sort()
        self.checkMatrix(mat)
        # Delete some column.
        del self.lp.cols[2]
        mat = [(r, c, v) for r, c, v in mat if c < 2] + [
            (r, c-1, v) for r, c, v in mat if c > 2]
        mat.sort()
        self.checkMatrix(mat)

    def testNumberNonZeroAttribute(self):
        """Tests nnz in the LP and in vectors."""
        # Run the initial test.
        self.assertEqual(self.lp.nnz, 0)
        for row in self.lp.rows:
            self.assertEqual(row.nnz, 0)
        for col in self.lp.cols:
            self.assertEqual(col.nnz, 0)
        # Set the matrix.
        mat = self.matrix
        self.lp.matrix = mat
        self.assertEqual(self.lp.nnz, len(mat))
        self.assertEqual([r.nnz for r in self.lp.rows], [3, 3, 4])
        self.assertEqual([c.nnz for c in self.lp.cols], [2, 3, 3, 2])
        # Clear a row.
        del self.lp.rows[1].matrix
        self.assertEqual(self.lp.nnz, len(mat)-3)
        self.assertEqual([r.nnz for r in self.lp.rows], [3, 0, 4])
        self.assertEqual([c.nnz for c in self.lp.cols], [2, 2, 2, 1])
        # Delete a column outright.
        del self.lp.cols[-1]
        self.assertEqual(self.lp.nnz, len(mat)-4)
        self.assertEqual([r.nnz for r in self.lp.rows], [3, 0, 3])
        self.assertEqual([c.nnz for c in self.lp.cols], [2, 2, 2])
        # Clear the matrix.
        del self.lp.matrix
        self.assertEqual(self.lp.nnz, 0)
        self.assertEqual([r.nnz for r in self.lp.rows], [0, 0, 0])
        self.assertEqual([c.nnz for c in self.lp.cols], [0, 0, 0])

    def testMatrixSetBadValueTypesWithExplicitIndexing(self):
        """Test setting the explicitly-indexed matrix with bad value types."""
        with self.assertRaises(TypeError):
            self.lp.matrix = [(0, 1, 2), (0, 2, "hello"), (1, 3, [4])]

    def testMatrixSetBadValueTypesWithImplicitIndexing(self):
        """Test setting the implicitly-indexed matrix with bad value types."""
        with self.assertRaises(TypeError):
            self.lp.matrix = ["hello", [4]]

    def testMatrixSetVectorBadValueTypesWithExplicitIndexing(self):
        """Test setting explicitly-indexed vectors with bad value types."""
        with self.assertRaises(TypeError):
            self.lp.rows[1].matrix = [(0, 5), (1, int), (2, "hi!")]

    def testMatrixSetVectorBadValueTypesWithImplicitIndexing(self):
        """Test setting implicitly-indexed vectors with bad value types."""
        with self.assertRaises(TypeError):
            self.lp.rows[1].matrix = [5, int, "hi!"]
        with self.assertRaises(TypeError):
            self.lp.cols[1].matrix = [5, int, "hi!"]

    def testMatrixSetBadIndexTypes(self):
        """Test setting a matrix with bad index types."""
        with self.assertRaises(TypeError):
            self.lp.matrix = [(0, 1, 2), (0, {"hi": 2}, 3.1415), (1, 3, 4)]
        with self.assertRaises(TypeError):
            self.lp.matrix = [(0, 1, 2), (0, 2, 3.1415), (1, [3], 4)]

    def testVectorSetBadIndexTypes(self):
        """Test setting vectors with bad index types."""
        with self.assertRaises(TypeError):
            self.lp.rows[1].matrix = [(1, 2), ({"hi": 2}, 3.1415), (3, 4)]
        with self.assertRaises(TypeError):
            self.lp.cols[2].matrix = [(1, 2), (2, 3.1415), ([3], 4)]

    def testMatrixSetWithOutOfRangeExplicitIndexing(self):
        """Test setting the matrix with explicit indices out of range."""
        with self.assertRaises(IndexError):
            self.lp.matrix = [(0, 1, 2), (6, 6, 2)]
        with self.assertRaises(IndexError):
            self.lp.matrix = [(0, -5, 2), (0, 2, 3.1415), (1, 3, 4)]

    def testMatrixSetWithOutOfRangeImplicitIndexing(self):
        """Test setting the matrix with implicit indices out of range."""
        with self.assertRaises(IndexError):
            self.lp.matrix = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13]
        with self.assertRaises(IndexError):
            self.lp.matrix = [(2, 3, 4), 5]

    def testVectorSetWithOutOfRangeExplicitIndexing(self):
        """Test setting vectors with explicit indices out of range."""
        with self.assertRaises(IndexError):
            self.lp.rows[1].matrix = [(1, 2), (3, 4), (4, 8)]
        with self.assertRaises(IndexError):
            self.lp.cols[2].matrix = [(1, 2), (2, 3.1415), (3, 4)]

    def testVectorSetWithOutOfRangeImplicitIndexing(self):
        """Test setting vectors with implicit indices out of range."""
        with self.assertRaises(IndexError):
            self.lp.rows[1].matrix = [1, 2, 3, 4, 5]
        with self.assertRaises(IndexError):
            self.lp.cols[2].matrix = [(2, 3.1415), 1]

    def testMatrixSetWithDuplicateIndices(self):
        """Test setting the matrix with duplicate indices."""
        with self.assertRaises(ValueError):
            self.lp.matrix = [(0, 1, 2), (0, 1, 3)]
        with self.assertRaises(ValueError):
            self.lp.matrix = [(2, 0, 2), (1, 3, 3.1415), 4]

    def testVectorSetWithDuplicateIndices(self):
        """Test setting vector matrix entries with duplicate indices."""
        with self.assertRaises(ValueError):
            self.lp.rows[1].matrix = [(1, 2), (0, 4), 8]
        with self.assertRaises(ValueError):
            self.lp.cols[2].matrix = [(1, 2), (2, 3.1415), (1, 4)]


class MatrixScalingTestCase(unittest.TestCase):
    """Tests the scale and unscale functions."""
    def setUp(self):
        self.lp = glpk.LPX()
        self.lp.rows.add(100)
        self.lp.cols.add(100)
        rgen = random.Random(0)
        newmatrix = []
        # This should be pretty poorly conditioned.
        for row in self.lp.rows:
            scale = rgen.uniform(0, 1000)
            newmatrix.extend(rgen.uniform(-scale, scale) for c in self.lp.cols)
        self.lp.matrix = newmatrix

    def testScaling(self):
        """Tests default scaling with the LPX.scale method."""
        self.lp.scale()

    @unittest.skipUnless(
        glpk.env.version > (4, 31),
        'Parameterized scaling is only available as of GLPK 4.31.'
    )
    def testFlaggedScaling(self):
        """Tests scaling with flags."""
        self.lp.scale(glpk.LPX.SF_EQ | glpk.LPX.SF_2N)

    def testUnscaling(self):
        """Tests the LPX.unscale method."""
        self.lp.scale()
        self.lp.unscale()
        self.assertEqual(set([1.0]), set(r.scale for r in self.lp.rows) |
                         set(c.scale for c in self.lp.cols))



"""Tests for naming objects with .name attributes."""
import unittest
import glpk


class NameTestCase:
    """Constains test set methods for setting names.

    Note that these tests are not a unit test test case, but they can
    be inhereted by a test case to test the 'nameability' of various
    objects within GLPK.

    In order to use this series of tests, one would assign the
    object's attribute 'nameable' in the setUp() method to the object
    one should treat as having an assignable 'name' attribute."""
    def testNameInitiallyNone(self):
        """Tests that a name is initially None."""
        self.assertEqual(self.nameable.name, None)

    def testNameSet(self):
        """Tests that setting a name sticks."""
        self.nameable.name = 'a simple name'
        self.assertEqual(self.nameable.name, 'a simple name')

    def testNameReset(self):
        """Tests that setting a name unsets the old name."""
        self.nameable.name = 'albatross'
        self.nameable.name = 'new names? '
        self.assertEqual(self.nameable.name, 'new names? ')

    def testNameSetNone(self):
        """Tests unsetting a name by setting it to None."""
        self.nameable.name = 'foo'
        self.nameable.name = None
        self.assertEqual(self.nameable.name, None)

    def testNameDelete(self):
        """Tests unsetting a name by using a del statement."""
        self.nameable.name = 'foo'
        del self.nameable.name
        self.assertEqual(self.nameable.name, None)

    def testNameSetToBadType(self):
        """Tests that setting non-string names raises TypeErrors."""
        def f(newName): self.nameable.name = newName
        self.assertRaises(TypeError, f, 4)
        self.assertRaises(TypeError, f, ['foo'])
        self.assertRaises(TypeError, f, {'foo': 'bar'})


class LPNameTestCase(unittest.TestCase, NameTestCase):
    """Naming tests for the linear program name."""
    def setUp(self):
        self.lp = glpk.LPX()
        self.nameable = self.lp


class ObjectiveNameTestCase(unittest.TestCase, NameTestCase):
    """Naming tests for the objective function name."""
    def setUp(self):
        self.lp = glpk.LPX()
        self.nameable = self.lp.obj


class RowNameTestCase(unittest.TestCase, NameTestCase):
    """Naming tests for a row name."""
    def setUp(self):
        self.lp = glpk.LPX()
        rnew = self.lp.rows.add(1)
        self.nameable = self.lp.rows[rnew]


class ColumnNameTestCase(unittest.TestCase, NameTestCase):
    """Naming tests for a column name."""
    def setUp(self):
        self.lp = glpk.LPX()
        cnew = self.lp.cols.add(1)
        self.nameable = self.lp.cols[cnew]



"""Tests for the solver itself."""

from glpk import env, LPX
import sys
import unittest
from itertools import cycle


class SimpleSolverTest(unittest.TestCase):
    """A simple suite of tests for this problem.

    max (x+y) subject to
    0.5*x + y <= 1
    0 <= x <= 1
    0 <= y <= 1

    This should have optimal solution x=1, y=0.5."""
    def setUp(self):
        lp = self.lp = LPX()
        lp.rows.add(1)
        lp.cols.add(2)
        lp.cols[0].name = 'x'
        lp.cols[1].name = 'y'
        for c in lp.cols:
            c.bounds = 0, 1
        lp.obj[:] = [1, 1]
        lp.obj.maximize = True
        lp.rows[0].matrix = [0.5, 1.0]
        lp.rows[0].bounds = None, 1

    def testSimplex(self):
        """Tests solving the simple problem with simplex."""
        self.lp.simplex()
        self.assertAlmostEqual(self.lp.cols['x'].value, 1.0)
        self.assertAlmostEqual(self.lp.cols['y'].value, 0.5)
        self.assertAlmostEqual(self.lp.obj.value_s, 1.5)

    def testExact(self):
        """Tests solving the simple problem with exact."""
        self.lp.exact()
        self.assertEqual(self.lp.cols['x'].value, 1.0)
        self.assertEqual(self.lp.cols['y'].value, 0.5)

    def testInterior(self):
        """Tests solving the simple problem with interior."""
        self.lp.interior()
        self.assertAlmostEqual(self.lp.cols['x'].value, 1.0)
        self.assertAlmostEqual(self.lp.cols['y'].value, 0.5)

    def testSimplexKKT(self):
        """Tests the KKT check with solution from simplex solver."""
        # Solve test LP using the simplex method,
        # check KKT conditions on optimal solution
        self.lp.simplex()
        kkt = self.lp.kkt()

        # The test LP has an optimal solution.
        # The KKT conditions should be (approximately) satisfied.
        self.assertAlmostEqual(kkt.pe_ae_max, 0.0)
        self.assertAlmostEqual(kkt.pe_re_max, 0.0)
        self.assertAlmostEqual(kkt.pb_ae_max, 0.0)
        self.assertAlmostEqual(kkt.pb_re_max, 0.0)
        self.assertAlmostEqual(kkt.de_ae_max, 0.0)
        self.assertAlmostEqual(kkt.de_re_max, 0.0)
        self.assertAlmostEqual(kkt.db_ae_max, 0.0)
        self.assertAlmostEqual(kkt.db_re_max, 0.0)

        # Are the row/column/variable indices valid?
        # In the test LP, there are:
        # - 1 row
        # - 2 columns
        # - 3 variables (rows + columns)
        # If the error reported is exactly 0, then glp_check_kkt
        # returns an index of 0.

        # Primal equality constraints (one for each row)
        self.assertTrue(kkt.pe_ae_row in [0, 1])
        self.assertTrue(kkt.pe_re_row in [0, 1])

        # Primal bound constraints (one for each variable)
        # There are one row and two columns in the test LP.
        self.assertTrue(kkt.pb_ae_ind in [0, 1, 2, 3])
        self.assertTrue(kkt.pb_re_ind in [0, 1, 2, 3])

        # Dual equality constraints (one for each column)
        self.assertTrue(kkt.de_ae_row in [0, 1, 2])
        self.assertTrue(kkt.de_re_row in [0, 1, 2])

        # Dual bound constraints (one for each variable)
        self.assertTrue(kkt.db_ae_ind in [0, 1, 2, 3])
        self.assertTrue(kkt.db_re_ind in [0, 1, 2, 3])

        # Are the KKT "quality" values valid?
        self.assertTrue(kkt.pe_quality in ['H', 'M', 'L', '?'])
        self.assertTrue(kkt.pb_quality in ['H', 'M', 'L', '?'])
        self.assertTrue(kkt.de_quality in ['H', 'M', 'L', '?'])
        self.assertTrue(kkt.db_quality in ['H', 'M', 'L', '?'])

    def testExactKKT(self):
        """Tests the KKT check with solution from exact solver."""
        # Solve test LP using the simplex method,
        # check KKT conditions on optimal solution
        self.lp.exact()
        kkt = self.lp.kkt()

        # The test LP has an optimal solution.
        # The KKT conditions should be (approximately) satisfied.
        self.assertAlmostEqual(kkt.pe_ae_max, 0.0)
        self.assertAlmostEqual(kkt.pe_re_max, 0.0)
        self.assertAlmostEqual(kkt.pb_ae_max, 0.0)
        self.assertAlmostEqual(kkt.pb_re_max, 0.0)
        self.assertAlmostEqual(kkt.de_ae_max, 0.0)
        self.assertAlmostEqual(kkt.de_re_max, 0.0)
        self.assertAlmostEqual(kkt.db_ae_max, 0.0)
        self.assertAlmostEqual(kkt.db_re_max, 0.0)

        # Are the row/column/variable indices valid?
        # In the test LP, there are:
        # - 1 row
        # - 2 columns
        # - 3 variables (rows + columns)
        # If the error reported is exactly 0, then glp_check_kkt
        # returns an index of 0.

        # Primal equality constraints (one for each row)
        self.assertTrue(kkt.pe_ae_row in [0, 1])
        self.assertTrue(kkt.pe_re_row in [0, 1])

        # Primal bound constraints (one for each variable)
        # There are one row and two columns in the test LP.
        self.assertTrue(kkt.pb_ae_ind in [0, 1, 2, 3])
        self.assertTrue(kkt.pb_re_ind in [0, 1, 2, 3])

        # Dual equality constraints (one for each column)
        self.assertTrue(kkt.de_ae_row in [0, 1, 2])
        self.assertTrue(kkt.de_re_row in [0, 1, 2])

        # Dual bound constraints (one for each variable)
        self.assertTrue(kkt.db_ae_ind in [0, 1, 2, 3])
        self.assertTrue(kkt.db_re_ind in [0, 1, 2, 3])

        # Are the KKT "quality" values valid?
        self.assertTrue(kkt.pe_quality in ['H', 'M', 'L', '?'])
        self.assertTrue(kkt.pb_quality in ['H', 'M', 'L', '?'])
        self.assertTrue(kkt.de_quality in ['H', 'M', 'L', '?'])
        self.assertTrue(kkt.db_quality in ['H', 'M', 'L', '?'])

    def testAdvBasis(self):
        # adding for coverage
        # TODO: how to test this?
        self.lp.adv_basis()
        self.assertTrue(self.lp.simplex() is None)
        self.assertEqual('opt', self.lp.status)

    def testCpxBasis(self):
        # adding for coverage
        # TODO: how to test this?
        self.lp.cpx_basis()
        self.assertTrue(self.lp.simplex() is None)
        self.assertEqual('opt', self.lp.status)


class SimpleIntegerSolverTest(unittest.TestCase):
    """A simple suite of tests for this problem.

    max (2*x+y) subject to
    0.5*x + y <= 1
    0 <= x <= 1
    0 <= y <= 1
    x, y integer

    This should have optimal solution x=1, y=0."""
    def setUp(self):
        lp = self.lp = LPX()
        lp.rows.add(1)
        lp.cols.add(2)
        lp.cols[0].name = 'x'
        lp.cols[1].name = 'y'
        for c in lp.cols:
            c.bounds = 0, 1
            c.kind = int
        lp.obj[:] = [2, 1]
        lp.obj.maximize = True
        lp.rows[0].matrix = [0.5, 1.0]
        lp.rows[0].bounds = None, 1

    def testInteger(self):
        """Tests solving the simple problem with integer."""
        class Callback:
            def default(self, tree):
                pass
        self.lp.integer(presolve=True, callback=Callback())
        self.assertAlmostEqual(self.lp.cols['x'].value, 1.0)
        self.assertAlmostEqual(self.lp.cols['y'].value, 0.0)
        self.assertAlmostEqual(self.lp.obj.value, 2.0)

    def testIntegerKKT(self):
        """Tests the KKT check with solution from integer solver."""
        # Solve test LP using the simplex method,
        # check KKT conditions on optimal solution
        self.lp.integer(presolve=True)
        kkt = self.lp.kktint()

        # The test IP has an optimal solution.
        # The primal KKT conditions should be (approximately) satisfied.
        self.assertAlmostEqual(kkt.pe_ae_max, 0.0)
        self.assertAlmostEqual(kkt.pe_re_max, 0.0)
        self.assertAlmostEqual(kkt.pb_ae_max, 0.0)
        self.assertAlmostEqual(kkt.pb_re_max, 0.0)

        # Are the row/column/variable indices valid?
        # In the test IP, there are:
        # - 1 row
        # - 2 columns
        # - 3 variables (rows + columns)
        # If the error reported is exactly 0, then glp_check_kkt
        # returns an index of 0.

        # Primal equality constraints (one for each row)
        self.assertTrue(kkt.pe_ae_row in [0, 1])
        self.assertTrue(kkt.pe_re_row in [0, 1])

        # Primal bound constraints (one for each variable)
        # There are one row and two columns in the test LP.
        self.assertTrue(kkt.pb_ae_ind in [0, 1, 2, 3])
        self.assertTrue(kkt.pb_re_ind in [0, 1, 2, 3])

        # Are the KKT "quality" values valid?
        self.assertTrue(kkt.pe_quality in ['H', 'M', 'L', '?'])
        self.assertTrue(kkt.pb_quality in ['H', 'M', 'L', '?'])

    def testIntOpt(self):
        """Test solving a simple integer program w/ intopt."""
        # FIXME: documentation claims that this method does not require an
        # existing LP relaxation
        self.lp.simplex()
        self.lp.intopt()
        self.assertEqual(self.lp.status, 'opt')
        self.assertAlmostEqual(self.lp.cols['x'].value, 1.0)
        self.assertAlmostEqual(self.lp.cols['y'].value, 0.0)


class TwoDimensionalTest(unittest.TestCase):
    def setUp(self):
        self.lp = LPX()

    def testEvolvingConstraintsSimplex(self):
        """Test repeatedly simplex solving an LP with evolving constraints."""
        lp = self.lp
        # Set up the rules of the problem.
        lp.cols.add(2)
        lp.obj[0, 1] = 1
        # Try very simple rules.
        x1, x2 = lp.cols[0], lp.cols[1]  # For convenience...
        x1.name, x2.name = 'x1', 'x2'
        x1.bounds = None, 1
        x2.bounds = None, 2
        lp.obj.maximize = True
        self.assertEqual(None, lp.simplex())
        self.assertEqual('opt', lp.status)
        self.assertAlmostEqual(lp.obj.value, 3)
        self.assertAlmostEqual(x1.primal, 1)
        self.assertAlmostEqual(x2.primal, 2)
        # Now try pushing it into unbounded territory.
        lp.obj[0] = -1
        self.assertEqual(None, lp.simplex())  # No error?
        self.assertEqual('unbnd', lp.status)
        # Redefine the bounds so it is bounded again.
        x1.bounds = -1, None
        self.assertEqual(None, lp.simplex())
        self.assertEqual('opt', lp.status)

        self.assertAlmostEqual(lp.obj.value, 3)
        self.assertAlmostEqual(x1.primal, -1)
        self.assertAlmostEqual(x2.primal, 2)
        # Now add in a row constraint forcing it to a new point, (1/2, 2).
        lp.rows.add(1)
        lp.rows[0].matrix = [-2, 1]
        lp.rows[0].bounds = None, 1
        self.assertEqual(None, lp.simplex())
        self.assertEqual('opt', lp.status)
        self.assertAlmostEqual(lp.obj.value, 1.5)
        self.assertAlmostEqual(x1.primal, .5)
        self.assertAlmostEqual(x2.primal, 2)
        # Now add in another forcing it to point (1/4, 3/2).
        lp.rows.add(1)
        lp.rows[-1].matrix = [-2, -1]
        lp.rows[-1].bounds = -2, None

        self.assertEqual(None, lp.simplex())
        self.assertEqual('opt', lp.status)

        self.assertAlmostEqual(lp.obj.value, 1.25)
        self.assertAlmostEqual(x1.primal, .25)
        self.assertAlmostEqual(x2.primal, 1.5)
        # Now go for the gusto.  Change column constraint, force infeasibility.
        x2.bounds = 2, None  # Instead of x2<=2, must now be >=2!  Tee hee.
        self.assertEqual(None, lp.simplex())
        self.assertEqual('nofeas', lp.status)
        # By removing the first row constraint, we allow opt point (-1, 1).
        del lp.rows[0]
        lp.std_basis()

        self.assertEqual(None, lp.simplex())
        self.assertEqual('opt', lp.status)

        self.assertAlmostEqual(lp.obj.value, 5)
        self.assertAlmostEqual(x1.primal, -1)
        self.assertAlmostEqual(x2.primal, 4)

    def testEvolvingConstraintsInterior(self):
        """Similar, but for the interior point method."""
        lp = self.lp
        # Set up the rules of the problem.
        lp.cols.add(2)
        lp.obj[0, 1] = 1
        # Try very simple rules.
        x1, x2 = lp.cols[0], lp.cols[1]  # For convenience...
        x1.name, x2.name = 'x1', 'x2'
        x1.bounds = None, 1
        x2.bounds = None, 2
        lp.obj.maximize = True
        # This should fail, since interior point methods need some
        # rows, and some columns.
        self.assertEqual('fail', lp.interior())
        # Now try pushing it into unbounded territory.
        lp.obj[0] = -1
        # Redefine the bounds so it is bounded again.
        x1.bounds = -1, None
        # Now add in a row constraint forcing it to a new point, (1/2, 2).
        lp.rows.add(1)
        lp.rows[0].matrix = [-2, 1]
        lp.rows[0].bounds = None, 1
        self.assertEqual(None, lp.interior())
        self.assertEqual('opt', lp.status)
        self.assertAlmostEqual(lp.obj.value, 1.5)
        self.assertAlmostEqual(x1.primal, .5)
        self.assertAlmostEqual(x2.primal, 2)
        # Now add in another forcing it to point (1/4, 3/2).
        lp.rows.add(1)
        lp.rows[-1].matrix = [-2, -1]
        lp.rows[-1].bounds = -2, None

        self.assertEqual(None, lp.interior())
        self.assertEqual('opt', lp.status)

        self.assertAlmostEqual(lp.obj.value, 1.25)
        self.assertAlmostEqual(x1.primal, .25)
        self.assertAlmostEqual(x2.primal, 1.5)
        # Now go for the gusto.  Change column constraint, force infeasibility.
        x2.bounds = 3, None  # Instead of x2<=2, must now be >=3!  Tee hee.
        self.assertEqual(None, lp.interior())
        self.assertEqual('nofeas', lp.status)
        # By removing the first row constraint, we allow opt point (-1, 4).
        del lp.rows[0]
        lp.std_basis()

        self.assertEqual(None, lp.interior())
        self.assertEqual('opt', lp.status)

        self.assertAlmostEqual(lp.obj.value, 5)
        self.assertAlmostEqual(x1.primal, -1)
        self.assertAlmostEqual(x2.primal, 4)


class SatisfiabilityMIPTest(unittest.TestCase):
    @classmethod
    def solve_sat(self, expression):
        """Attempts to satisfy a formula of conjunction of disjunctions.

        If there are n variables in the expression, this will return a
        list of length n, all elements booleans.  The truth of element i-1
        corresponds to the truth of variable i.

        If no satisfying assignment could be found, None is returned."""
        if len(expression) == 0:
            return []
        numvars = max(max(abs(v) for v in clause) for clause in expression)
        lp = LPX()
        lp.cols.add(2*numvars)
        for col in lp.cols:
            col.bounds = 0.0, 1.0

        def lit2col(lit):
            if lit > 0:
                return 2*lit-2
            return 2*(-lit)-1

        for i in range(1, numvars+1):
            lp.cols[lit2col(i)].name = 'x_%d' % i
            lp.cols[lit2col(-i)].name = '!x_%d' % i
            lp.rows.add(1)
            lp.rows[-1].matrix = [(lit2col(i), 1.0), (lit2col(-i), 1.0)]
            lp.rows[-1].bounds = 1.0
        for clause in expression:
            lp.rows.add(1)
            lp.rows[-1].matrix = [(lit2col(lit), 1.0) for lit in clause]
            lp.rows[-1].bounds = 1, None
        retval = lp.simplex()
        if retval is not None:
            return None
        if lp.status != 'opt':
            return None
        for col in lp.cols:
            col.kind = int
        retval = lp.integer()
        if retval is not None:
            return None
        if lp.status != 'opt':
            return None
        return [col.value > 0.99 for col in lp.cols[::2]]

    @classmethod
    def verify(self, expression, assignment):
        """Get the truth of an expression given a variable truth assignment.

        This will return true only if this is a satisfying assignment."""
        # Each clause must be true.
        for clause in expression:
            # For a disjunctive clause to be true, at least one of its
            # literals must be true.
            lits_true = 0
            for lit in clause:
                if (lit > 0) == (assignment[abs(lit)-1]):
                    lits_true += 1
            if lits_true == 0:
                return False
        return True

    def testSolvableSat(self):
        """Test that a solvable satisfiability problem can be solved."""
        exp = [(3, -2, 5), (1, 2, 5), (1, 3, -4), (-1, -5, -3), (-4, 2, 1),
               (5, 2, -1), (5, 2, 1), (-4, -1, -5), (4, 5, -1), (3, 1, -2),
               (1, 5, -3), (-5, -3, -1), (-4, -5, -3), (-3, -5, -2),
               (4, -5, 3), (1, -2, -5), (1, 4, -3), (4, -1, -3),
               (-5, -1, 3), (-2, -4, -5)]
        solution = self.solve_sat(exp)
        # First assert that this is a solution.
        self.assertNotEqual(solution, None)
        self.assertTrue(self.verify(exp, solution))

    def testInsolvableSat(self):
        """Test that an unsolvable satisfiability problem can't be solved."""
        exp = [(1, 2), (1, -2), (-1, 2), (-1, -2)]
        solution = self.solve_sat(exp)
        self.assertEqual(solution, None)


@unittest.skipIf(
    env.version < (4, 20),
    "callbacks did't exist prior to 4.20"
)
class MIPCallbackTest(unittest.TestCase):
    # This CNF expression is complicated enough to the point where
    # when run through our SAT solver, the resulting search tree will
    # have a fair number of calls, and the callback shall be called a
    # number of times.  Incidentally, it does have valid assignment
    # FFFTTTTFTFTFTTFFFFFTTTFFTTTTTT.
    expression = [
        (-5, -26, -23), (-20, 24, -3), (23, -1, 14), (28, -1, -17),
        (-13, -1, -7), (-7, 14, 9), (20, -6, -30), (22, 29, -13),
        (18, 27, -26), (-8, 24, 13), (21, 12, 14), (-12, 15, 1), (18, 12, 6),
        (17, 26, 7), (14, -9, 17), (25, 27, 23), (13, -2, 27), (15, 11, -17),
        (1, 7, 6), (24, 25, -8), (-1, 23, -8), (-3, -5, 16), (-14, -10, -15),
        (6, -4, 27), (25, -1, 5), (21, 17, 7), (7, -20, 12), (2, -9, 30),
        (-10, -29, -23), (27, 2, -25), (27, -30, -22), (-7, -21, 13),
        (9, 15, -10), (7, -10, -30), (-21, 26, 28), (15, -30, -8),
        (28, 7, -23), (11, 9, 27), (-5, -17, 4), (24, 25, -11), (-18, 7, 3),
        (28, 14, -9), (21, -3, -4), (-30, -13, 4), (4, 28, -12), (8, -15, -4),
        (-30, -9, -18), (-28, 30, 4), (-17, 21, -20), (8, -3, 9),
        (-20, -29, -12), (-26, 27, 10), (-18, -8, 1), (3, -20, 9),
        (-5, 16, -24), (-24, -30, 25), (-28, 9, -27), (10, 28, 25),
        (-21, 6, 13), (8, 2, 29), (-26, -14, -12), (-20, -13, -18),
        (-4, -8, 25), (8, 1, -16), (-21, -22, -8), (13, -17, 28), (2, -3, 16),
        (27, -10, 21), (23, 18, 26), (6, 8, -7), (21, -22, 11), (2, -30, 25),
        (29, 5, 24), (-14, 28, -30), (5, 10, -4), (-19, -13, 4), (-1, 6, 14),
        (26, -7, 9), (8, 21, 24), (15, -26, -24), (4, 25, 23), (30, -5, 16),
        (-11, -16, -1), (-12, 24, -21), (7, 1, 9), (16, -30, 14),
        (30, 10, -6), (-1, 12, 13), (23, 27, 15), (19, 13, 20),
        (-24, -25, 19), (27, -17, -5), (-2, -16, 23), (1, -16, 29),
        (-19, -7, 27), (-16, 12, 23), (-25, 9, -19), (-24, 27, 10),
        (26, -1, -27), (23, -24, 6), (14, -28, 22), (5, -22, 11),
        (-22, -10, -4), (6, -2, -18), (22, -25, 29), (-4, -21, -3),
        (-6, 7, -25), (22, 10, -18), (-24, 26, 10), (-12, 27, -23),
        (-5, -20, -2), (-26, 28, -2), (-9, 4, 3), (-21, 26, 20), (20, 8, -2),
        (-19, -17, -16), (6, 27, 13), (22, -23, 8), (26, -2, -3),
        (-30, -2, -16), (22, -27, 1), (3, -2, -12), (-24, -25, -26),
        (17, 10, 11), (4, 20, 14), (22, -17, -27), (-12, 16, -3),
        (-25, -20, -10), (23, -16, -5), (3, 30, 15), (25, 28, 5),
        (-24, 22, 23), (13, -14, -30), (-21, -17, 28), (-2, -3, -17),
        (-19, 16, 3), (-6, 26, 1), (9, 27, -18), (21, 17, 29), (30, 8, 7)
    ]

    allreasons = 'select prepro rowgen heur cutgen branch bingo'.split()

    @classmethod
    def solve_sat(self, expression=None, callback=None,
                  return_processor=None, kwargs={}):
        """Runs a solve sat with a callback.  This is similar to the
        other SAT solving procedures in this test suite, with some
        modifications to enable testing of the MIP callback
        functionality.

        expression is the expression argument, and defaults to the
        expression declared in the MIPCallbackTest.

        callback is the callback instance being tested.

        return_processor is a function called with the retval from the
        lp.integer call, and the lp, e.g., return_processor(retval,
        lp), prior to any further processing of the results.

        kwargs is a dictionary of other optional keyword arguments and
        their values which is passed to the lp.integer solution
        procedure."""
        if expression is None:
            expression = self.expression
        if len(expression) == 0:
            return []
        numvars = max(max(abs(v) for v in clause) for clause in expression)
        lp = LPX()
        lp.cols.add(2*numvars)
        for col in lp.cols:
            col.bounds = 0.0, 1.0

        def lit2col(lit):
            if lit > 0:
                return 2*lit-2
            return 2*(-lit)-1

        for i in range(1, numvars+1):
            lp.cols[lit2col(i)].name = 'x_%d' % i
            lp.cols[lit2col(-i)].name = '!x_%d' % i
            lp.rows.add(1)
            lp.rows[-1].matrix = [(lit2col(i), 1.0), (lit2col(-i), 1.0)]
            lp.rows[-1].bounds = 1.0
        for clause in expression:
            lp.rows.add(1)
            lp.rows[-1].matrix = [(lit2col(lit), 1.0) for lit in clause]
            lp.rows[-1].bounds = 1, None
        retval = lp.simplex()
        if retval is not None:
            return None
        if lp.status != 'opt':
            return None
        for col in lp.cols:
            col.kind = int
        retval = lp.integer(callback=callback, **kwargs)
        if return_processor:
            return_processor(retval, lp)
        if retval is not None:
            return None
        if lp.status != 'opt':
            return None
        return [col.value > 0.99 for col in lp.cols[::2]]

    @classmethod
    def verify(self, expression, assignment):
        """Get the truth of an expression given a variable truth assignment.

        This will return true only if this is a satisfying assignment."""
        # Each clause must be true.
        for clause in expression:
            # For a disjunctive clause to be true, at least one of its
            # literals must be true.
            lits_true = 0
            for lit in clause:
                if (lit > 0) == (assignment[abs(lit)-1]):
                    lits_true += 1
            if lits_true == 0:
                return False
        return True

    def testEmptyCallback(self):
        """Tests for errors with an empty callback."""
        class Callback:
            pass
        assign = self.solve_sat(callback=Callback())
        self.assertTrue(self.verify(self.expression, assign))

    def testUncallableCallback(self):
        """Tests that there is an error with an uncallable callback."""
        class Callback:
            default = "Hi, I'm a string!"
        self.Callback = Callback
        with self.assertRaises(TypeError):
            self.solve_sat(callback=self.Callback())

    def testBadCallback(self):
        """Tests that errors in the callback are propagated properly."""
        testobj = self

        class Callback:
            def __init__(self):
                self.stopped = False

            def default(self, tree):
                # We should not be here if we have already made an error.
                testobj.assertFalse(self.stopped)
                self.stopped = True
                1 / 0

        self.Callback = Callback
        with self.assertRaises(ZeroDivisionError):
            self.solve_sat(callback=self.Callback())

    def testCallbackReasons(self):
        """Tests that there are no invalid Tree.reason codes."""
        reasons = set()

        class Callback:
            def default(self, tree):
                reasons.add(tree.reason)

        assign = self.solve_sat(callback=Callback())
        self.assertTrue(self.verify(self.expression, assign))

        # We should have some items.
        self.assertNotEqual(len(reasons), 0)
        # The reasons should not include anything other than those
        # reasons listed in the allreasons class member.
        self.assertTrue(reasons.issubset(set(self.allreasons)))

    def testCallbackMatchsReasons(self):
        """Ensure that callback methods and reasons are properly matched."""
        testobj = self

        class Callback:
            def select(self, tree): testobj.assertEqual(tree.reason, 'select')

            def prepro(self, tree): testobj.assertEqual(tree.reason, 'prepro')

            def branch(self, tree): testobj.assertEqual(tree.reason, 'branch')

            def rowgen(self, tree): testobj.assertEqual(tree.reason, 'rowgen')

            def heur(self, tree): testobj.assertEqual(tree.reason, 'heur')

            def cutgen(self, tree): testobj.assertEqual(tree.reason, 'cutgen')

            def bingo(self, tree): testobj.assertEqual(tree.reason, 'bingo')

            def default(self, tree): testobj.fail('should not reach default')
        assign = self.solve_sat(callback=Callback())
        self.assertTrue(self.verify(self.expression, assign))

    def testCallbackTerminate(self):
        """Tests that termination actually stops the solver."""
        testobj = self

        class Callback:
            def __init__(self):
                self.stopped = False

            def default(self, tree):
                # We should not be here if we have terminated.
                testobj.assertFalse(self.stopped)
                self.stopped = True
                tree.terminate()

        def rp(retval, lp):
            self.assertEqual(retval, 'stop')

        assign = self.solve_sat(callback=Callback(), return_processor=rp)
        self.assertEqual(assign, None)

    def testNodeCountsConsistent(self):
        """Tests that the tree node counts are consistent."""
        testobj = self

        class Callback:
            def __init__(self):
                self.last_total = 0

            def default(self, tree):
                testobj.assertTrue(tree.num_active <= tree.num_all)
                testobj.assertTrue(tree.num_all <= tree.num_total)
                testobj.assertTrue(self.last_total <= tree.num_total)
                testobj.assertEqual(tree.num_active, len(list(tree)))
                self.last_total = tree.num_total

        assign = self.solve_sat(callback=Callback())
        self.assertTrue(self.verify(self.expression, assign))

    @unittest.skipIf(env.version < (4, 21), "TODO")
    def testTreeSelectCallable(self):
        """Test that Tree.select is callable within select, not elsewhere."""
        testobj = self

        class Callback:
            def __init__(self):
                self.last_total = 0

            def select(self, tree):
                tree.select(tree.best_node)

            def default(self, tree):
                try:
                    tree.select(tree.best_node)
                except RuntimeError:
                    return
                else:
                    testobj.fail('Was able to call without error!')

        assign = self.solve_sat(callback=Callback())
        self.assertTrue(self.verify(self.expression, assign))

    def testSelectCurrNodeNone(self):
        """Test that there is no current node in the select callback."""
        testobj = self

        class Callback:
            def __init__(self):
                self.last_total = 0

            def select(self, tree):
                testobj.assertEqual(tree.curr_node, None)
        assign = self.solve_sat(callback=Callback())
        self.assertTrue(self.verify(self.expression, assign))

    def testTreeIterator(self):
        """Test the tree iterator."""
        testobj = self

        class Callback:
            def default(self, tree):
                explicit_actives = []
                n = tree.first_node
                while n:
                    explicit_actives.append(n.subproblem)
                    n = n.next
                actives = [node.subproblem for node in tree]
                testobj.assertEqual(actives, explicit_actives)

        assign = self.solve_sat(callback=Callback())
        self.assertTrue(self.verify(self.expression, assign))

    def testTreeNodeRichCompare(self):
        testobj = self

        class Callback:
            def default(self, tree):
                n = tree.first_node
                testobj.assertFalse(n == 1)
                testobj.assertFalse(n != 1)
                testobj.assertEqual(NotImplemented, n.__lt__(1))

                testobj.assertTrue(n == n)
                testobj.assertFalse(n != n)
                testobj.assertFalse(n < n)
                testobj.assertFalse(n > n)
                testobj.assertTrue(n <= n)
                testobj.assertTrue(n >= n)

        assign = self.solve_sat(callback=Callback())
        self.assertTrue(self.verify(self.expression, assign))

    def testTreeNodeAttrs(self):
        testobj = self
        tree_node_fmt_str = \
            "<glpk.TreeNode, active subprob {0:d} of glpk.Tree 0x{1:02x}"

        class Callback:
            def default(self, tree):
                n = tree.first_node
                testobj.assertTrue(n.active)
                testobj.assertGreaterEqual(n.level, 0)
                if n.level == 0:
                    testobj.assertTrue(n.up is None)
                else:
                    testobj.assertFalse(n.up is None)

                best = tree.best_node
                testobj.assertGreaterEqual(n.bound, best.bound)

                testobj.assertIn(
                    tree_node_fmt_str.format(n.subproblem, id(tree)),
                    str(n)
                )

        assign = self.solve_sat(callback=Callback())
        self.assertTrue(self.verify(self.expression, assign))

    def testTreeSelect(self):
        testobj = self

        class Callback:
            def select(self, tree):
                testobj.assertTrue(tree.select(tree.first_node) is None)

        assign = self.solve_sat(callback=Callback())
        self.assertTrue(self.verify(self.expression, assign))

    def testTreeBranch(self):
        testobj = self
        self.select_flags = cycle([None, b'D', b'U', b'N'])

        class Callback:
            def default(self, tree):
                if tree.reason != 'branch':
                    with testobj.assertRaises(RuntimeError) as cm:
                        tree.can_branch(1)
                    testobj.assertIn(
                        "function may only be called during branch phase",
                        str(cm.exception)
                    )
                    with testobj.assertRaises(RuntimeError) as cm:
                        tree.branch_upon(1)
                    testobj.assertIn(
                        "function may only be called during branch phase",
                        str(cm.exception)
                    )

            def branch(self, tree):
                testobj.assertEqual(type(tree.can_branch(1)), bool)

                with testobj.assertRaises(IndexError) as cm:
                    tree.can_branch(0)
                testobj.assertIn(
                    "index 0 out of bound for 60 columns",
                    str(cm.exception)
                )

                with testobj.assertRaises(IndexError) as cm:
                    tree.can_branch(61)
                testobj.assertIn(
                    "index 61 out of bound for 60 columns",
                    str(cm.exception)
                )

                with testobj.assertRaises(TypeError) as cm:
                    tree.branch_upon({})
                testobj.assertIn(
                    "an integer is required",
                    str(cm.exception)
                )

                with testobj.assertRaises(TypeError) as cm:
                    tree.branch_upon(1, {})
                if sys.version_info > (3,):
                    err_msg = "must be a byte string of length 1, not dict"
                else:
                    err_msg = "must be char, not dict"
                testobj.assertIn(
                    err_msg,
                    str(cm.exception)
                )

                with testobj.assertRaises(IndexError) as cm:
                    tree.branch_upon(0)
                testobj.assertIn(
                    "index 0 out of bound for 60 columns",
                    str(cm.exception)
                )
                with testobj.assertRaises(IndexError) as cm:
                    tree.branch_upon(61)
                testobj.assertIn(
                    "index 61 out of bound for 60 columns",
                    str(cm.exception)
                )
                # find a column that we can't branch on and then try to
                # branch on it
                try:
                    idx = next(
                        i for i in range(1, 61) if not tree.can_branch(i)
                    )
                    with testobj.assertRaises(RuntimeError) as cm:
                        tree.branch_upon(idx)
                    testobj.assertIn(
                        "cannot branch upon this column",
                        str(cm.exception)
                    )
                except StopIteration:
                    pass

                branch_idx = next(
                    i for i in range(1, 61) if tree.can_branch(i)
                )
                with testobj.assertRaises(ValueError) as cm:
                    tree.branch_upon(branch_idx, b'x')
                testobj.assertIn(
                    "select argument must be D, U, or N",
                    str(cm.exception)
                )

                select_flag = next(testobj.select_flags)
                if select_flag is None:
                    tree.branch_upon(branch_idx)
                else:
                    tree.branch_upon(branch_idx, select_flag)

        assign = self.solve_sat(callback=Callback())
        self.assertTrue(self.verify(self.expression, assign))

    def testTreeHeuristic(self):
        testobj = self

        class Callback:
            def default(self, tree):
                if tree.reason != 'heur':
                    with testobj.assertRaises(RuntimeError) as cm:
                        tree.heuristic(1)
                    testobj.assertIn(
                        "function may only be called during heur phase",
                        str(cm.exception)
                    )

            def heur(self, tree):
                with testobj.assertRaises(TypeError) as cm:
                    tree.heuristic(1)
                testobj.assertIn(
                    "'int' object is not iterable",
                    str(cm.exception)
                )
                # FIXME: this iterator has only 1 object
                with testobj.assertRaises(ValueError) as cm:
                    tree.heuristic([1])
                testobj.assertIn(
                    "iterator had only 2 objects, but 60 required",
                    str(cm.exception)
                )
                with testobj.assertRaises(TypeError) as cm:
                    tree.heuristic(['a'])
                testobj.assertIn(
                    "iterator must return floats",
                    str(cm.exception)
                )

                assignment = [
                    0.0, 1.0, 0.0, 1.0, 0.0, 1.0, 1.0, 0.0, 1.0, 0.0, 0.0, 1.0,
                    1.0, 0.0, 1.0, 0.0, 1.0, 0.0, 0.0, 1.0, 1.0, 0.0, 1.0, 0.0,
                    1.0, 0.0, 1.0, 0.0, 1.0, 0.0, 0.0, 1.0, 0.0, 1.0, 0.0, 1.0,
                    0.0, 1.0, 0.0, 1.0, 0.0, 1.0, 1.0, 0.0, 1.0, 0.0, 0.0, 1.0,
                    1.0, 0.0, 0.0, 1.0, 1.0, 0.0, 1.0, 0.0, 1.0, 0.0, 1.0, 0.0
                ]
                accepted = tree.heuristic(assignment)
                testobj.assertEqual(type(accepted), bool)
                testobj.assertTrue(accepted)

        assign = self.solve_sat(callback=Callback())
        self.assertTrue(self.verify(self.expression, assign))

    def testTreeGap(self):
        testobj = self
        self.old_gap = float('inf')

        class Callback:
            def default(self, tree):
                new_gap = tree.gap
                testobj.assertEqual(type(new_gap), float)
                testobj.assertGreaterEqual(new_gap, 0.0)
                testobj.assertLessEqual(new_gap, testobj.old_gap)
                testobj.old_gap = new_gap

        assign = self.solve_sat(callback=Callback())
        self.assertTrue(self.verify(self.expression, assign))



"""Tests for setting parameters."""


from glpk import env, LPX
import random
import sys
import unittest


class ParamsTestCase(unittest.TestCase):
    """Tests for setting and getting parameters of the LP."""
    def setUp(self):
        self.lp = LPX()
        param_names = [n for n in dir(self.lp.params) if n[0] != '_']
        param_names.remove('reset')
        self.param_name2default = dict([
            (n, getattr(self.lp.params, n)) for n in param_names])

        # INTEGER PARAMETER VALUES

        # Have nice lists of, for each parameter, valid values, and
        # values illegal because they are of bad type or value.
        p2v, p2bt, p2bv = {}, {}, {}
        # These have legal values 0, 1
        for n in 'price'.split():
            p2v[n] = 0, 1
            p2bv[n] = -100, -1, 2, 3, 4, 5
        # These have legal values 0, 1,2
        for n in 'branch mpsobj'.split():
            p2v[n] = 0, 1, 2
        # These have legal values 0, 1,2, 3
        for n in 'msglev scale btrack'.split():
            p2v[n] = 0, 1, 2, 3
        # These all have the following illegal typed values.
        illegals = 'hi!', [1], {2: 5}, 0.5, complex(1, 0), self.lp
        for n in p2v:
            p2bt[n] = illegals
        # Set their illegal values.
        illegal_vals = -100, -1, 2, 3, 4, 5, 1000
        for n in p2v:
            legals = set(p2v[n])
            p2bv[n] = tuple([v for v in illegal_vals if v not in legals])
        # These may be any positive integer.
        for n in 'outfrq'.split():
            p2v[n] = 1, 2, 3, 100, 1600, 80000, 4000000, 2000000000
            p2bt[n] = illegals
            p2bv[n] = 0, -1, -2, -3, -100, -100000
        # These may be any integer.
        for n in 'itlim'.split():
            p2v[n] = illegal_vals
            p2bt[n] = illegals
        # Integer in range 255.
        for n in 'usecuts'.split():
            p2v[n] = range(0, 0x100)
            p2bt[n] = illegals
            p2bv[n] = tuple(range(-10, 0)) + tuple(range(0x100, 0x110))

        # FLOAT PARAMETER VALUES

        # Previous illegal types included floats.
        illegals = tuple([iv for iv in illegals if type(iv) != float])
        # Floats without bound.
        for n in 'objll objul outdly tmlim'.split():
            p2v[n] = -1e100, -5.23e20, -2.231, -1, 0, 0.5, 1, 3.14159, 1e100
            p2bt[n] = illegals
        # Bounded in [0, 1] range.
        for n in 'relax'.split():
            p2v[n] = 0, .2, .45678, .901, 1
            p2bv[n] = -5e6, -1, -0.01, 1.01, 1.5, 1000
            p2bt[n] = illegals
        # Bounded in double-epsilon to 0.001 range.
        for n in 'tolbnd toldj tolint tolobj tolpiv'.split():
            p2v[n] = 2.3e-16, 1e-8, 0.000523, 0.001
            p2bv[n] = -1000, -1e-8, 0, 0.0011, 0.1, 5.123, 1e100
            p2bt[n] = illegals

        # BOOLEAN PARAMETER VALUES

        # These have boolean values.  They have no illegal values
        # owing to PyGLPK's pecularity of having boolean values set to
        # the bool(value) of value, which is always defined.
        bname = 'dual mpsfree mpsinfo mpsorig mpsskip mpswide presol round'
        for n in bname.split():
            p2v[n] = False, True

        # Set up the mapping from parameter names to tuples of legal
        # values, and illegal params because of value and type.
        self.param_name2values = p2v
        self.param_name2bad_values = p2bv
        self.param_name2bad_type_values = p2bt

    def testIterationCountReadOnly(self):
        """The itcnt parameter should be read only."""
        # TypeError was raised in pre-2.5 versions of Python.
        # AttributeError is raised in 2.5-post versions of Python.
        with self.assertRaises(Exception):
            self.lp.params.itcnt = 5
        with self.assertRaises(Exception):
            self.lp.params.itcnt = -300

    def testDefaultSetIsNotIllegal(self):
        """Sets all parameters to their default values."""
        p2dv = dict(self.param_name2default)
        del p2dv['itcnt']
        for pname, pvalue in p2dv.items():
            setattr(self.lp.params, pname, pvalue)

    def listOfParameterPairs(self, pname2pvalues):
        """Reduce a parameter name, value-sequence mapping to a list.

        Given a mapping from parameter names to a parameter value
        sequence, construct a list of all parameter names and
        parameter values."""
        pp = [(name, value) for name, values in pname2pvalues.items()
              for value in values]
        return pp

    def testSetValues(self):
        """Set many parameter values."""
        pp = self.listOfParameterPairs(self.param_name2values)
        random.Random(2).shuffle(pp)
        for n, v in pp:
            setattr(self.lp.params, n, v)
            self.assertEqual(getattr(self.lp.params, n), v)

    def testSetValuesThenReset(self):
        """Set many parameter values, then use reset() to set defaults."""
        pp = self.listOfParameterPairs(self.param_name2values)
        pp = [(n, v) for n, v in pp if self.param_name2default[n] != v]
        random.Random(3).shuffle(pp)
        for n, v in pp:
            setattr(self.lp.params, n, v)
        self.lp.params.reset()
        for n, v in sorted(self.param_name2default.items()):
            self.assertEqual(getattr(self.lp.params, n), v)

    def testSetBadValues(self):
        """Set many parameters values of bad value."""
        pp = self.listOfParameterPairs(self.param_name2bad_values)
        random.Random(4).shuffle(pp)
        for n, v in pp:
            self.assertRaises(ValueError, setattr, self.lp.params, n, v)

    def testSetBadTypeValues(self):
        """Set many parameters values of bad type."""
        pp = self.listOfParameterPairs(self.param_name2bad_type_values)
        random.Random(5).shuffle(pp)
        for n, v in pp:
            self.assertRaises(TypeError, setattr, self.lp.params, n, v)


try:
    Params
except NameError:
    del ParamsTestCase


class IntegerProgramTestCase(unittest.TestCase):
    """Tests for setting the program type (continuous/int)."""
    def setUp(self):
        self.lp = LPX()

    def testDefaultKind(self):
        """Test that the default kind is float (continuous)."""
        self.assertEqual(self.lp.kind, float)
        self.lp.cols.add(5)
        self.assertEqual([c.kind for c in self.lp.cols], [float]*5)
        self.lp.rows.add(4)
        self.assertEqual([r.kind for r in self.lp.rows], [float]*4)

    def testSettingKind(self):
        """Test that the kind of the problem is not mutable."""
        target_error = AttributeError
        if sys.hexversion < 0x02050000:
            target_error = TypeError
        self.assertRaises(target_error, setattr, self.lp, 'kind', int)
        self.assertRaises(target_error, setattr, self.lp, 'kind', float)

    def testSettingColumnsInt(self):
        """Test setting columns as int."""
        self.lp.cols.add(5)
        for c in [-1, 1, 2]:
            self.lp.cols[c].kind = int
        self.assertEqual([c.kind for c in self.lp.cols],
                         [float, int, int, float, int])

    def testIntAndBinaryCounters(self):
        """Test nint and nbin counters."""
        self.lp.cols.add(5)

        # Convenience function to check the values of nint and nbin.
        def nn(ni, nb):
            self.assertEqual((self.lp.nint, self.lp.nbin), (ni, nb))

        # Do a series of ops and watch how nint and nbin changes.
        nn(0, 0)
        self.lp.cols[0].kind = int
        nn(1, 0)
        self.lp.cols[4].kind = int
        nn(2, 0)
        self.lp.cols[4].bounds = 0, 2
        nn(2, 0)
        self.lp.cols[-1].kind = int
        nn(2, 0)
        self.lp.cols[0].kind = float
        nn(1, 0)
        self.lp.cols[3].kind = int
        nn(2, 0)
        self.lp.cols[4].bounds = 0, 1
        nn(2, 1)
        self.lp.cols[1].kind = int
        nn(3, 1)
        self.lp.cols[3].bounds = 0, 1
        nn(3, 2)
        self.lp.cols[4].bounds = None
        nn(3, 1)
        del self.lp.cols[3]
        nn(2, 0)
        self.lp.cols[2].bounds = 0, 1
        nn(2, 0)
        self.lp.cols[2].kind = int
        nn(3, 1)

    def testIntBounds(self):
        """Tests setting various rows and columns as integer or boolean."""
        self.lp.cols.add(3)
        x, y, z = tuple(self.lp.cols)

        def tk(*targetkinds):
            """Test kind convenience function."""
            self.assertEqual(targetkinds, tuple(c.kind for c in self.lp.cols))
            if set(targetkinds) == set([float]):
                pk = float
            else:
                pk = int
            self.assertEqual(pk, self.lp.kind)

        tk(float, float, float)
        x.kind = int
        x.bounds = 0, 5
        tk(int, float, float)
        y.kind = bool
        tk(int, bool, float)
        y.bounds = 0, 4
        tk(int, int, float)
        x.bounds = 0, 1
        tk(bool, int, float)
        x.kind = float
        tk(float, int, float)
        y.kind = float
        tk(float, float, float)

    def testSettingsRowsInt(self):
        """Test setting rows as int, which should always fail."""
        self.lp.rows.add(5)
        with self.assertRaises(ValueError):
            self.lp.rows[2].kind = int
        with self.assertRaises(ValueError):
            self.lp.rows[4].kind = int

    def testSettingsRowsWithBadObject(self):
        """Test setting columns as something neither int or float."""
        self.lp.cols.add(5)
        for kind in [complex, 'belay', 2, {1: 3}]:
            with self.assertRaises(ValueError):
                self.lp.cols[2].kind = kind


class SimplexControlParametersTestCase(unittest.TestCase):
    """Tests for the simplex method's control parameters."""
    def setUp(self):
        lp = self.lp = LPX()
        lp.rows.add(1)
        lp.cols.add(2)
        for c in lp.cols:
            c.bounds = 0, 1
        lp.obj[:] = [1, 1]
        lp.obj.maximize = True
        lp.rows[0].matrix = [0.5, 1.0]
        lp.rows[0].bounds = None, 1
        self.num_illegals = 100

    def runValueErrorTest(self, param_name, legals):
        """Runs a test of valid values."""
        valid_values = set(legals)
        # Start with "boundary" values.
        values = [min(valid_values)-1, max(valid_values)+1]
        # Come up with a lot of illegal values.
        while len(values) < self.num_illegals:
            randint = random.randint(-20000, 20000)
            if randint not in valid_values:
                values.append(randint)
        for value in values:
            with self.assertRaises(ValueError):
                self.lp.simplex(**{param_name: value})

    def testMessageLevel(self):
        """Test the msg_lev parameter."""
        # For obvious reasons, we only try no output unfortunately.
        self.lp.simplex(msg_lev=LPX.MSG_OFF)

    def testMessageLevelValueErrors(self):
        """Test whether illegal values for msg_lev throw exceptions."""
        self.runValueErrorTest("msg_lev", [
            LPX.MSG_OFF, LPX.MSG_ERR,
            LPX.MSG_ON, LPX.MSG_ALL
        ])

    def testSimplexMethod(self):
        """Test the meth parameter."""
        if env.version >= (4, 31):
            legals = LPX.PRIMAL, LPX.DUAL, LPX.DUALP
        else:
            legals = LPX.PRIMAL, LPX.DUALP
        for p in legals:
            self.lp.simplex(meth=p)

    def testSimplexMethodValueErrors(self):
        """Test whether illegal values for meth throw exceptions."""
        if env.version >= (4, 31):
            legals = LPX.PRIMAL, LPX.DUAL, LPX.DUALP
        else:
            legals = LPX.PRIMAL, LPX.DUALP
        self.runValueErrorTest("meth", legals)

    def testPricingTechnique(self):
        """Test the pricing parameter."""
        for p in (LPX.PT_STD, LPX.PT_PSE):
            self.lp.simplex(pricing=p)

    def testPricingTechniqueValueErrors(self):
        """Test whether illegal values for pricing throw exceptions."""
        self.runValueErrorTest("pricing", [
            LPX.PT_STD, LPX.PT_PSE])

    def testRatioTestTechnique(self):
        """Test the r_test parameter."""
        for p in (LPX.RT_STD, LPX.RT_HAR):
            self.lp.simplex(pricing=p)

    def testRatioTestTechniqueValueErrors(self):
        """Test whether illegal values for r_test throw exceptions."""
        self.runValueErrorTest("r_test", [
            LPX.RT_STD, LPX.RT_HAR])

    def testTolerancePrimalFeasible(self):
        """Test the tol_bnd parameter."""
        for p in (1e-6, 1e-7, 1e-1, .99):
            self.lp.simplex(tol_bnd=p)

    def testTolerancePrimalFeasibleValueErrors(self):
        """Test whether illegal values for tol_bnd throw exceptions."""
        for p in (-1, -1e6, 0, 1, 1e1, 1e5):
            with self.assertRaises(ValueError):
                self.lp.simplex(tol_bnd=p)

    def testToleranceDualFeasible(self):
        """Test the tol_dj parameter."""
        for p in (1e-6, 1e-7, .99):
            self.lp.simplex(tol_dj=p)

    def testToleranceDualFeasibleValueErrors(self):
        """Test whether illegal values for tol_dj throw exceptions."""
        for p in (-1, -1e6, 0, 1, 1e1, 1e5):
            with self.assertRaises(ValueError):
                self.lp.simplex(tol_dj=p)

    def testTolerancePivotalFeasible(self):
        """Test the tol_piv parameter."""
        for p in (1e-6, 1e-7, 1e-1, .99):
            self.lp.simplex(tol_piv=p)

    def testTolerancePivotalFeasibleValueErrors(self):
        """Test whether illegal values for tol_piv throw exceptions."""
        for p in (-1, -1e6, 0, 1, 1e1, 1e5):
            with self.assertRaises(ValueError):
                self.lp.simplex(tol_piv=p)

    def testObjectiveLowerLimit(self):
        """Test the obj_ll parameter."""
        for p in (-200, -100, 0, 3.14159, 100, 200):
            self.lp.simplex(obj_ll=p)

    def testObjectiveLowerLimitTypeErrors(self):
        """Test whether illegal types for obj_ll throw exceptions."""
        for p in ("foo", {"bar": "biz"}, set("baz"), complex(2, 3)):
            with self.assertRaises(TypeError):
                self.lp.simplex(obj_ll=p)

    def testObjectiveUpperLimit(self):
        """Test the obj_ul parameter."""
        for p in (-200, -100, 0, 3.14159, 100, 200):
            self.lp.simplex(obj_ul=p)

    def testObjectiveUpperLimitTypeErrors(self):
        """Test whether illegal types for obj_ul throw exceptions."""
        for p in ("foo", {"bar": "biz"}, set("baz"), complex(2, 3)):
            with self.assertRaises(TypeError):
                self.lp.simplex(obj_ul=p)

    def testIterationLimit(self):
        """Test the it_lim parameter."""
        for p in (0, 100, 200, 1000):
            self.lp.simplex(it_lim=p)

    def testIterationLimitValueErrors(self):
        """Test whether illegal values for it_lim throw exceptions."""
        for p in (-1, -100, -200, -1000000):
            with self.assertRaises(ValueError):
                self.lp.simplex(it_lim=p)

    def testTimeLimit(self):
        """Test the tm_lim parameter."""
        for p in (0, 100, 200, 1000):
            self.lp.simplex(tm_lim=p)

    def testTimeLimitValueErrors(self):
        """Test whether illegal values for tm_lim throw exceptions."""
        for p in (-1, -100, -200, -1000000):
            with self.assertRaises(ValueError):
                self.lp.simplex(tm_lim=p)

    def testOutputFrequency(self):
        """Test the out_frq parameter."""
        for p in (1, 100, 200, 1000):
            self.lp.simplex(out_frq=p)

    def testOutputFrequencyValueErrors(self):
        """Test whether illegal values for out_frq throw exceptions."""
        for p in (0, -1, -100, -200, -1000000):
            with self.assertRaises(ValueError):
                self.lp.simplex(out_frq=p)

    def testOutputDelay(self):
        """Test the out_dly parameter."""
        for p in (0, 1, 100, 200, 1000):
            self.lp.simplex(out_dly=p)

    def testOutputDelayValueErrors(self):
        """Test whether illegal values for out_dly throw exceptions."""
        for p in (-1, -100, -200, -1000000):
            with self.assertRaises(ValueError):
                self.lp.simplex(out_dly=p)

    def testPresolver(self):
        """Test the presolve parameter."""
        self.lp.simplex(presolve=False)
        self.lp.simplex(presolve=True)

    def testPresolverTypeErrors(self):
        """Test whether illegal types for obj_ll throw exceptions."""
        for p in ("foo", {"bar": "biz"}, set("baz"), complex(2, 3)):
            with self.assertRaises(TypeError):
                self.lp.simplex(presolve=p)


@unittest.skipIf(
    env.version < (4, 20),
    'Integer control parameters did not exist prior to GLPK 4.20.'
)
class IntegerControlParametersTestCase(unittest.TestCase):
    """Tests for the simplex method's control parameters.

    Note that one control parameter, callback, is not tested here, but
    rather in the solve test suite.  For GLPK 4.33 many of these options
    are not really tested since GLPK 4.33 appears to be seriously broken
    with respect to parameter setting."""
    def setUp(self):
        lp = self.lp = LPX()
        lp.rows.add(2)
        lp.cols.add(2)
        for c in lp.cols:
            c.bounds = None, None
            c.kind = int
        lp.obj[:] = [1, 1]
        lp.obj.maximize = True
        lp.rows[0].matrix, lp.rows[1].matrix = [2, 1], [1, 2]
        lp.rows[0].bounds, lp.rows[1].bounds = (None, 6.5), (None, 6.5)
        lp.simplex()  # This should have both column vars at 2.1666...
        self.num_illegals = 100

    def runValueErrorTest(self, param_name, legals):
        """Runs a test of valid values."""
        valid_values = set(legals)
        # Start with "boundary" values.
        values = [min(valid_values)-1, max(valid_values)+1]
        # Come up with a lot of illegal values.
        while len(values) < self.num_illegals:
            randint = random.randint(-20000, 20000)
            if randint not in valid_values:
                values.append(randint)
        for value in values:
            with self.assertRaises(ValueError):
                self.lp.integer(**{param_name: value})

    def testMessageLevel(self):
        """Test the msg_lev parameter."""
        # For obvious reasons, we only try no output unfortunately.
        if env.version == (4, 33):
            return
        self.lp.integer(msg_lev=LPX.MSG_OFF)

    def testMessageLevelValueErrors(self):
        """Test whether illegal values for msg_lev throw exceptions."""
        self.runValueErrorTest("msg_lev", [
            LPX.MSG_OFF, LPX.MSG_ERR,
            LPX.MSG_ON, LPX.MSG_ALL
        ])

    def testBranchingTechnique(self):
        """Test the br_tech parameter."""
        if env.version == (4, 33):
            return
        legals = (
            LPX.BR_FFV, LPX.BR_LFV, LPX.BR_MFV,
            LPX.BR_DTH, LPX.BR_PCH
        )
        for p in legals:
            self.lp.integer(br_tech=p)

    def testBranchingTechniqueValueErrors(self):
        """Test whether illegal values for br_tech throw exceptions."""
        self.runValueErrorTest("br_tech", [
            LPX.BR_FFV, LPX.BR_LFV, LPX.BR_MFV,
            LPX.BR_DTH, LPX.BR_PCH
        ])

    def testBacktrackingTechnique(self):
        """Test the bt_tech parameter."""
        if env.version == (4, 33):
            return
        legals = (
            LPX.BT_DFS, LPX.BT_BFS,
            LPX.BT_BLB, LPX.BT_BPH
        )
        for p in legals:
            self.lp.integer(bt_tech=p)

    def testBacktrackingTechniqueValueErrors(self):
        """Test whether illegal values for bt_tech throw exceptions."""
        self.runValueErrorTest("bt_tech", [
            LPX.BT_DFS, LPX.BT_BFS, LPX.BT_BLB, LPX.BT_BPH
        ])

    def testPreprocessingTechnique(self):
        """Test the pp_tech parameter."""
        if env.version < (4, 21) or env.version == (4, 33):
            return
        legals = LPX.PP_NONE, LPX.PP_ROOT, LPX.PP_ALL
        for p in legals:
            self.lp.integer(pp_tech=p)

    def testPreprocessingTechniqueValueErrors(self):
        """Test whether illegal values for pp_tech throw exceptions."""
        if env.version < (4, 21):
            return
        self.runValueErrorTest("pp_tech", [
            LPX.PP_NONE, LPX.PP_ROOT, LPX.PP_ALL])

    def testGomorysMixedCuts(self):
        """Test the gmi_cuts option."""
        if env.version < (4, 24) or env.version == (4, 33):
            return
        legals = True, False
        for p in legals:
            self.lp.integer(gmi_cuts=p)

    def testGomorysMixedCutsTypeErrors(self):
        """Test whether illegal types for gmi_cuts throw exceptions."""
        for p in ("foo", {"bar": "biz"}, set("baz"), complex(2, 3)):
            with self.assertRaises(TypeError):
                self.lp.integer(gmi_cuts=p)

    def testMixedIntegerRoundingCuts(self):
        """Test the mir_cuts option."""
        if env.version < (4, 23) or env.version == (4, 33):
            return
        legals = True, False
        for p in legals:
            self.lp.integer(mir_cuts=p)

    def testMixedIntegerRoundingCutsTypeErrors(self):
        """Test whether illegal types for mir_cuts throw exceptions."""
        for p in ("foo", {"bar": "biz"}, set("baz"), complex(2, 3)):
            with self.assertRaises(TypeError):
                self.lp.integer(mir_cuts=p)

    def testToleranceIntegerFeasible(self):
        """Test the tol_int option."""
        if env.version == (4, 33):
            return
        legals = 1e-7, 1e-5, 1e-3, 1e-1, .99
        for p in legals:
            self.lp.integer(tol_int=p)

    def testToleranceIntegerFeasibleValueErrors(self):
        """Test whether illegal values for tol_obj throw exceptions."""
        for p in (-1, -1e6, 0, 1, 1e1, 1e5):
            with self.assertRaises(ValueError):
                self.lp.integer(tol_int=p)

    def testToleranceObjective(self):
        """Test the tol_obj option."""
        if env.version == (4, 33):
            return
        legals = 1e-7, 1e-5, 1e-3, 1e-1, .99
        for p in legals:
            self.lp.integer(tol_obj=p)

    def testToleranceObjectiveValueErrors(self):
        """Test whether illegal values for tol_obj throw exceptions."""
        for p in (-1, -1e6, 0, 1, 1e1, 1e5):
            with self.assertRaises(ValueError):
                self.lp.integer(tol_obj=p)

    def testTimeLimit(self):
        """Test the tm_lim parameter."""
        if env.version == (4, 33):
            return
        for p in (0, 100, 200, 1000):
            self.lp.integer(tm_lim=p)

    def testTimeLimitValueErrors(self):
        """Test whether illegal values for tm_lim throw exceptions."""
        for p in (-1, -100, -200, -1000000):
            with self.assertRaises(ValueError):
                self.lp.integer(tm_lim=p)

    def testOutputFrequency(self):
        """Test the out_frq parameter."""
        if env.version == (4, 33):
            return
        for p in (1, 100, 200, 1000):
            self.lp.integer(out_frq=p)

    def testOutputFrequencyValueErrors(self):
        """Test whether illegal values for out_frq throw exceptions."""
        for p in (0, -1, -100, -200, -1000000):
            with self.assertRaises(ValueError):
                self.lp.integer(out_frq=p)

    def testOutputDelay(self):
        """Test the out_dly parameter."""
        if env.version == (4, 33):
            return
        for p in (0, 1, 100, 200, 1000):
            self.lp.integer(out_dly=p)

    def testOutputDelayValueErrors(self):
        """Test whether illegal values for out_dly throw exceptions."""
        for p in (-1, -100, -200, -1000000):
            with self.assertRaises(ValueError):
                self.lp.integer(out_dly=p)



import glpk


def hamiltonian(edges):
    # maps node to col indices of incident edges
    node2colnums = {}
    for colnum, edge in enumerate(edges):
        n1, n2 = edge
        node2colnums.setdefault(n1, []).append(colnum)
        node2colnums.setdefault(n2, []).append(colnum)

    # empty LP instance
    lp = glpk.LPX()

    # stop annoying messages
    glpk.env.term_on = False

    # a struct var for each edge
    lp.cols.add(len(edges))
    # constraint for each node
    lp.rows.add(len(node2colnums) + 1)

    # make all struct variables binary
    for col in lp.cols:
        col.kind = bool

    # for each node, select at least 1 and at most 2 incident edges
    for row, edge_column_nums in zip(lp.rows, node2colnums.values()):
        row.matrix = [(cn, 1.0) for cn in edge_column_nums]
        row.bounds = 1, 2

    # we should select exactly (number of nodes - 1) edges total
    lp.rows[-1].matrix = [1.0] * len(lp.cols)
    lp.rows[-1].bounds = len(node2colnums) - 1

    # should not fail this way
    assert lp.simplex() is None
    # if no relaxed sol., no exact sol.
    if lp.status != 'opt':
        return None

    # should not fail this way
    assert lp.integer() is None
    # could not find integer solution
    if lp.status != 'opt':
        return None

    # return edges whose associated struct var has value 1
    return [edge for edge, col in zip(edges, lp.cols) if col.value > 0.99]

"""
1----2----3----5
      \  /
       \/  Has one H path!
       4

Path:
[(1, 2), (3, 4), (4, 2), (3, 5)]
"""

g1 = [(1, 2), (2, 3), (3, 4), (4, 2), (3, 5)]
print(hamiltonian(g1))

"""
4    5    6
|    |    |
|    |    | Has no H path!
1----2----3

Path:
None
"""

g2 = [(1, 2), (2, 3), (1, 4), (2, 5), (3, 6)]
print(hamiltonian(g2))

"""
4    5----6
|    |    |
|    |    | Has two H paths!
1----2----3

Path:
[(1, 2), (1, 4), (2, 5), (3, 6), (5, 6)]
"""

g3 = g2 + [(5, 6)]
print(hamiltonian(g3))



import glpk

def hamiltonian(edges):
    node2colnums = {} # Maps node to col indices of incident edges.
    for colnum, edge in enumerate(edges):
        n1, n2 = edge
        node2colnums.setdefault(n1, []).append(colnum)
        node2colnums.setdefault(n2, []).append(colnum)

    lp = glpk.LPX()                     # A new empty linear program
    glpk.env.term_on = False            # Stop annoying messages.
    lp.cols.add(len(edges))             # A struct var for each edge
    lp.rows.add(len(node2colnums)+1)    # Constraint for each node

    for col in lp.cols:                 # Go over all struct variables
        col.kind = bool                 # Make binary, not continuous

    # For each node, select at least 1 and at most 2 incident edges.
    for row, edge_column_nums in zip(lp.rows, node2colnums.values()):
        row.matrix = [(cn, 1.0) for cn in edge_column_nums]
        row.bounds = 1, 2

    # We should select exactly (number of nodes - 1) edges total
    lp.rows[-1].matrix = [1.0]*len(lp.cols)
    lp.rows[-1].bounds = len(node2colnums)-1

    assert lp.simplex() is None         # Should not fail this way.
    if lp.status != 'opt': return None  # If no relaxed sol., no exact sol.

    # Return the edges whose associated struct var has value 1.
    return [edge for edge, col in zip(edges, lp.cols) if col.value > 0.99]


#  1----2----3----5
#        \  /
#         \/  Has one H path!
#         4

g1 = [(1,2), (2,3), (3,4), (4,2), (3,5)]
print(hamiltonian(g1))

#  4    5    6
#  |    |    |
#  |    |    | Has no H path!
#  1----2----3

g2 = [(1,2), (2,3), (1,4), (2,5), (3,6)]
print(hamiltonian(g2))

#  4    5----6
#  |    |    |
#  |    |    | Has two H paths!
#  1----2----3

g3 = g2 + [(5,6)]
print(hamiltonian(g3))


def tsp(edges):
    node2colnums = {} # Maps node to col indices of incident edges.
    for colnum, edge in enumerate(edges):
        n1, n2, cost = edge
        node2colnums.setdefault(n1, []).append(colnum)
        node2colnums.setdefault(n2, []).append(colnum)

    lp = glpk.LPX()                     # A new empty linear program
    glpk.env.term_on = False            # Stop annoying messages.
    lp.cols.add(len(edges))             # A struct var for each edge
    lp.rows.add(len(node2colnums)+1)    # Constraint for each node

    lp.obj[:] = [e[-1] for e in edges]  # Try to minimize the total costs.
    lp.obj.maximize = False

    for col in lp.cols:                 # Go over all struct variables
        col.bounds = 0, 1               # Either edge selected (1) or not (0)
        col.kind = int                  # Make binary, not continuous

    # For each node, select two edges, i.e.., an arrival and a departure.
    for row, edge_column_nums in zip(lp.rows, node2colnums.values()):
        row.matrix = [(cn, 1.0) for cn in edge_column_nums]
        row.bounds = 2

    # We should select exactly (number of nodes) edges total
    lp.rows[-1].matrix = [1.0]*len(lp.cols)
    lp.rows[-1].bounds = len(node2colnums)

    assert lp.simplex() is None         # Should not fail this way.
    if lp.status != 'opt': return None  # If no relaxed sol., no exact sol.

    assert lp.integer() is None         # Should not fail this way.
    if lp.status != 'opt': return None  # Count not find integer solution!

    # Return the edges whose associated struct var has value 1.
    return [edge for edge, col in zip(edges, lp.cols) if col.value > 0.99]



"""Tests for changing the objective function."""

import glpk
import unittest


class ObjectiveDirectionTestCase(unittest.TestCase):
    """Tests whether setting this LP to max/min works.

    Note that these tests do not actually test any sort of
    optimization subject to maximization or minimization.  These tests
    merely check that setting the problem to maximization or
    minimization has the desirable properties of persistence."""
    def setUp(self):
        self.lp = glpk.LPX()

    def testInitiallyTrueOrFalse(self):
        """Tests whether the objective is initially boolean."""
        self.assertEqual(type(self.lp.obj.maximize), bool)

    def testMakeMaximize(self):
        """Tests making the LP a maximiziation problem."""
        self.lp.obj.maximize = True
        self.assertEqual(self.lp.obj.maximize, True)

    def testMakeMinimize(self):
        """Tests making the LP a minimization problem."""
        self.lp.obj.maximize = False
        self.assertEqual(self.lp.obj.maximize, False)


class ObjectiveShiftTestCase(unittest.TestCase):
    """Tests setting the objective's constant shift coefficient."""
    def setUp(self):
        self.lp = glpk.LPX()

    def testInitialShiftAttribute(self):
        """Tests that the initial shift lp.obj.shift is 0."""
        self.assertEqual(self.lp.obj.shift, 0.0)
        self.assertEqual(self.lp.obj[None], 0.0)

    def testSetShiftThroughAttribute(self):
        """Tests our ability to set shift through lp.obj.shift."""
        self.lp.obj.shift = 5.23
        self.assertEqual(self.lp.obj.shift, 5.23)

    def testSetShiftThroughIndex(self):
        """Tests our ability to set shift through lp.obj[None]."""
        self.lp.obj[None] = 2.345
        self.assertEqual(self.lp.obj[None], 2.345)

    def testShiftIndexAndAttributeEquivalent(self):
        """Tests equivalence of lp.obj[None] and lp.obj.shift."""
        self.lp.obj.shift = -12.345
        self.assertEqual(self.lp.obj[None], -12.345)
        self.lp.obj[None] = 3.14159
        self.assertEqual(self.lp.obj.shift, 3.14159)

    def testBadTypeAssignment(self):
        """Tests assigning objects other than a float as shift."""
        with self.assertRaises(TypeError):
            self.lp.obj.shift = None
        with self.assertRaises(TypeError):
            self.lp.obj[None] = "Foo"
        with self.assertRaises(TypeError):
            self.lp.obj.shift = [1]
        with self.assertRaises(TypeError):
            self.lp.obj[None] = {1: 2}


class ObjectiveCoefficientTestCase(unittest.TestCase):
    """Tests setting and indexing objective function coefficients."""
    def setUp(self):
        self.lp = glpk.LPX()
        self.lp.cols.add(5)

    def testObjectiveIterator(self):
        obj_iter = iter(self.lp.obj)
        self.assertEqual(len(obj_iter), 5)
        next(obj_iter)
        self.assertEqual(len(obj_iter), 4)

    def testInitialCoefficients(self):
        """Test that the initial coefficients are 0."""
        for i in range(len(self.lp.cols)):
            self.assertEqual(self.lp.obj[i], 0.0)
        # While we're at it, run the same test with the iterator
        # functionality of the objective.
        self.assertEqual([c for c in self.lp.obj], [0.0]*len(self.lp.cols))
        self.assertEqual(self.lp.obj[:], [0.0]*len(self.lp.cols))

    def testSetSingleCoefficient(self):
        """Test setting a single coefficient."""
        self.lp.obj[1] = 2
        self.assertEqual(self.lp.obj[1], 2.0)
        self.assertEqual(self.lp.obj[:], [0.0, 2.0, 0.0, 0.0, 0.0])

    def testSetTupleIndexToManyCoefficients(self):
        """Tests setting a tuple of coefficients to many coefficients."""
        self.lp.obj[0, -1, 3] = 3.14159, -20, 8
        self.assertEqual(self.lp.obj[:], [3.14159, 0.0, 0.0, 8.0, -20])
        self.assertEqual(self.lp.obj[0, -1, 3], [3.14159, -20, 8])
        self.assertEqual(self.lp.obj[0, None], [3.14159, self.lp.obj.shift])

    def testSetTupleIndexToSingleCoefficient(self):
        """Tests setting a tuple of coefficients to a single coefficient."""
        self.lp.obj[0, -1, 3] = 2.3
        self.assertEqual(self.lp.obj[:], [2.3, 0.0, 0.0, 2.3, 2.3])

    def testSetSliceIndexToManyCoefficients(self):
        """Tests setting a slice of coefficients to many coefficients."""
        self.lp.obj[1::2] = 1, 2
        self.assertEqual(self.lp.obj[:], [0.0, 1.0, 0.0, 2.0, 0.0])

    def testSetSliceIndexToSingleCoefficient(self):
        """Tests setting a slice of coefficients to a single coefficient."""
        self.lp.obj[1::2] = 0.5
        self.assertEqual(self.lp.obj[:], [0.0, 0.5, 0.0, 0.5, 0.0])

    def testStringIndex(self):
        """Tests setting objective coefficients by col name."""
        for c in self.lp.cols:
            c.name = 'x%i' % c.index
        # Test single value assignment.
        self.lp.obj['x1'] = 3.1
        self.assertEqual(self.lp.obj['x1'], 3.1)
        self.assertEqual(self.lp.obj[:], [0.0, 3.1, 0.0, 0.0, 0.0])
        # Test tuple value assignment.
        self.lp.obj['x1', 3, 'x0'] = 123, 456, 789
        self.assertEqual(self.lp.obj[:], [789, 123, 0.0, 456, 0.0])

    def testSetSingleCoefficientBadType(self):
        """Test setting a single coefficient with a bad type."""
        with self.assertRaises(TypeError):
            self.lp.obj[1] = None
        with self.assertRaises(TypeError):
            self.lp.obj[3] = [2]
        with self.assertRaises(TypeError):
            self.lp.obj[-1] = {1: 2}

    def testSetMultipleCoefficientBadType(self):
        """Tests setting multiple coefficients with a bad type."""
        with self.assertRaises(TypeError):
            self.lp.obj[0, 1] = 2, None
        with self.assertRaises(TypeError):
            self.lp.obj[-2:] = 2, [1]
        with self.assertRaises(TypeError):
            self.lp.obj[-2:] = self

    def testSetBadNumberOfCoefficientsToTuple(self):
        """Test a mismatch between assignees and assignments."""
        with self.assertRaises(ValueError):
            self.lp.obj[1, 2, 3] = 4, 5, 6, 7
        with self.assertRaises(ValueError):
            self.lp.obj[1, 2, 3] = 4, 5
        # Tuple indexing for good measure.
        with self.assertRaises(ValueError):
            self.lp.obj[-2:] = 1, 4, 5



"""Tests for basic functionality"""

import tempfile
import sys
import unittest

from glpk import LPX

MODEL = '''
var x >= 0, <= 1;
var y >= 0, <= 1;

param a;
param b;

maximize sum: x + y;
s.t. foo: a * x + y <= 1;

end;
'''

DATA = '''
param b := 0.5;
'''


class BarColTests(unittest.TestCase):

    def setUp(self):
        self.lp = LPX()
        self.lp.cols.add(5)
        self.lp.rows.add(5)

    def testBarColIter(self):
        row_iter = iter(self.lp.rows)
        self.assertEqual(len(row_iter), 5)
        next(row_iter)
        self.assertEqual(len(row_iter), 4)

    @unittest.skipIf(
        sys.version_info > (3,),
        "rich comparison for BarCol is under development"
    )
    def testBarColRichCompare(self):
        cols = self.lp.cols
        rows = self.lp.rows

        self.assertFalse(cols == 1)
        self.assertTrue(cols != 1)
        self.assertFalse(cols < 1)
        self.assertFalse(cols <= 1)
        self.assertTrue(cols > 1)
        self.assertTrue(cols >= 1)

        self.assertTrue(cols == cols)
        self.assertFalse(cols == rows)
        self.assertFalse(cols != cols)
        self.assertTrue(cols != rows)
        self.assertFalse(cols < cols)
        self.assertTrue(cols <= cols)
        self.assertFalse(cols > cols)
        self.assertTrue(cols >= cols)

        # these comparison's depend on the id of the object which changes order
        # between runs
        self.assertEqual(cols < rows, cols <= rows)
        self.assertNotEqual(cols <= rows, cols >= rows)
        self.assertEqual(cols > rows, cols >= rows)

    def testBarColDelete(self):
        with self.assertRaises(KeyError) as cm:
            del self.lp.cols[0, 'x']
        self.assertIn("col named 'x' does not exist", str(cm.exception))
        with self.assertRaises(ValueError) as cm:
            del self.lp.cols[0, 1, 0]
        self.assertIn("duplicate index detected", str(cm.exception))
        with self.assertRaises(TypeError) as cm:
            del self.lp.cols[0.1]
        self.assertIn(
            "bad index type for row/col collection",
            str(cm.exception)
        )

    def testBarColAdd(self):
        self.lp.rows.add(3)
        self.lp.cols[0] = [2, 4, 6]
        self.assertTrue(len(self.lp.matrix), 3)
        self.assertEqual(self.lp.matrix[1], (1, 0, 4.0))
        with self.assertRaises(TypeError) as cm:
            self.lp.cols[0.1] = [2, 4, 6]
        self.assertIn(
            "bad index type for row/col collection",
            str(cm.exception)
        )

    def testBarColStr(self):
        lp_id = id(self.lp)
        self.assertIn(
            "<glpk.BarCollection, cols of glpk.LPX 0x{0:02x}".format(lp_id),
            str(self.lp.cols)
        )

    def testBarRichCompare(self):
        c0 = self.lp.cols[0]
        c1 = self.lp.cols[1]

        self.assertFalse(c0 == 0)
        self.assertFalse(c0 != 0)

        self.assertEqual(NotImplemented, c0.__lt__(1))

        self.assertTrue(c0 <= c1)
        self.assertFalse(c0 >= c1)

    def testBarStr(self):
        lp_id = id(self.lp)
        self.assertIn(
            "glpk.Bar, col 0 of glpk.LPX 0x{0:02x}".format(lp_id),
            str(self.lp.cols[0])
        )

    def testBarSetName(self):
        self.lp.cols[0].name = 'a' * 255
        with self.assertRaises(ValueError) as cm:
            self.lp.cols[0].name = 'a' * 256
        self.assertIn("name may be at most 255 chars", str(cm.exception))

    def testBarValid(self):
        c = self.lp.cols[-1]
        self.assertTrue(c.valid)
        del(self.lp.cols[-1])
        self.assertFalse(c.valid)

    def testBarIsCol(self):
        c = self.lp.cols[0]
        r = self.lp.rows[0]
        self.assertTrue(c.iscol)
        self.assertFalse(r.iscol)

    def testBarStatus(self):
        c = self.lp.cols[0]

        with self.assertRaises(ValueError) as cm:
            c.status = 'xxx'
        self.assertIn(
            "status strings must be length 2",
            str(cm.exception)
        )

        with self.assertRaises(ValueError) as cm:
            c.status = 'xx'
        self.assertIn(
            "status string value 'xx' unrecognized",
            str(cm.exception)
        )

        c.status = 'bs'
        self.assertEqual(c.status, 'bs')
        c.status = 'ns'
        self.assertEqual(c.status, 'ns')

        c.bounds = None, None
        c.status = 'nf'
        self.assertEqual(c.status, 'nf')

        c.bounds = 0, 1
        c.status = 'nl'
        self.assertEqual(c.status, 'nl')
        c.status = 'nu'
        self.assertEqual(c.status, 'nu')

        with self.assertRaises(AttributeError) as cm:
            del(c.status)
        self.assertIn("cannot delete status", str(cm.exception))

    def testBarKind(self):
        c = self.lp.cols[0]

        self.assertEqual(c.kind, float)
        c.kind = int
        self.assertEqual(c.kind, int)
        c.kind = bool
        self.assertEqual(c.kind, bool)

        r = self.lp.rows[0]
        with self.assertRaises(ValueError) as cm:
            r.kind = bool
        self.assertIn("row variables cannot be binary", str(cm.exception))

    def testBarSpecVars(self):
        c = self.lp.cols[0]
        self.assertEqual(c.primal_s, 0.0)
        self.assertEqual(c.dual_s, 0.0)

        # TODO: need to actually test w/ MIP
        with self.assertRaises(TypeError) as cm:
            c.value_m
        self.assertIn(
            "MIP values require mixed integer problem",
            str(cm.exception)
        )

    def testBarSetMatrix(self):
        with self.assertRaises(ValueError) as cm:
            self.lp.cols[0].matrix = [(1, 2, 3)]
        self.assertIn(
            'vector entry tuple has length 3; 2 is required',
            str(cm.exception)
        )


class LpxTests(unittest.TestCase):

    def testInit(self):
        with self.assertRaises(TypeError) as cm:
            LPX(gmp='', mps='')
        self.assertIn(
            'cannot specify multiple data sources',
            str(cm.exception)
        )

        with self.assertRaises(RuntimeError) as cm:
            LPX(gmp=('', None, ''))
        self.assertIn(
            'GMP model reader failed',
            str(cm.exception)
        )

        model = tempfile.NamedTemporaryFile(mode='w')
        model.write(MODEL)
        model.flush()

        data = tempfile.NamedTemporaryFile(mode='w')
        data.write(DATA)
        data.flush()

        with self.assertRaises(RuntimeError) as cm:
            LPX(gmp=(model.name, ''))
        self.assertIn(
            'GMP data reader failed',
            str(cm.exception)
        )

        with self.assertRaises(RuntimeError) as cm:
            LPX(gmp=(model.name, data.name))
        self.assertIn(
            'GMP generator failed',
            str(cm.exception)
        )

    def testLpxStr(self):
        lp = LPX()
        self.assertIn(
            '<glpk.LPX 0-by-0 at 0x{0:02x}>'.format(id(lp)),
            str(lp)
        )

    def testLpxErase(self):
        lp = LPX()
        lp.cols.add(2)
        lp.rows.add(2)
        self.assertEqual(len(lp.cols), 2)
        self.assertEqual(len(lp.rows), 2)
        lp.erase()
        self.assertEqual(len(lp.cols), 0)
        self.assertEqual(len(lp.rows), 0)

    def testLpxSetMatrix(self):
        lp = LPX()
        with self.assertRaises(ValueError) as cm:
            lp.matrix = [(1, 2, 3, 4)]
        self.assertIn(
            'matrix entry tuple has length 4; 3 is required',
            str(cm.exception)
        )

    def testLpxInteger(self):
        lp = LPX()

        with self.assertRaises(RuntimeError) as cm:
            lp.integer()
        self.assertIn(
            'integer solver without presolve requires existing optimal basic '
            'solution',
            str(cm.exception)
        )

        with self.assertRaises(ValueError) as cm:
            lp.integer(ps_tm_lim=-1, presolve=True)
        self.assertIn('ps_tm_lim must be nonnegative', str(cm.exception))

        with self.assertRaises(ValueError) as cm:
            lp.integer(mip_gap=-1, presolve=True)
        self.assertIn('mip_gap must be non-negative', str(cm.exception))

    def testLpxKkt(self):
        lp = LPX()

        with self.assertRaises(RuntimeError) as cm:
            lp.kkt()
        self.assertIn(
            'cannot get KKT when primal or dual basic solution undefined',
            str(cm.exception)
        )

    def testLpxName(self):
        lp = LPX()
        with self.assertRaises(ValueError) as cm:
            lp.name = 'a' * 256
        self.assertIn('name may be at most 255 chars', str(cm.exception))

    def testLpxStatus(self):
        lp = LPX()
        self.assertEqual('undef', lp.status_s)

    # TODO: how to write a test where the ray returned is a row?
    def testLpxRay(self):
        lp = LPX()
        self.assertTrue(lp.ray is None)
        lp.cols.add(2)
        x1, x2 = lp.cols[0:2]
        lp.obj[:] = [-1.0, 1.0]
        lp.rows.add(2)
        r1, r2 = lp.rows[0:2]
        r1.matrix = [1.0, -1.0]
        r1.bounds = (-3, None)
        r2.matrix = [-1.0, 2.0]
        r2.bounds = (-2.0, None)
        x1.bounds = None
        x2.bounds = None
        lp.simplex()

        self.assertEqual(lp.status, 'unbnd')
        self.assertTrue(lp.ray.iscol)


class ObjectiveTests(unittest.TestCase):

    def testObjectiveShift(self):
        lp = LPX()
        with self.assertRaises(TypeError) as cm:
            del(lp.obj.shift)
        self.assertIn('deletion not supported', str(cm.exception))

    def testObjectiveAssignment(self):
        lp = LPX()
        with self.assertRaises(TypeError) as cm:
            del(lp.obj[0])
        self.assertIn(
            "objective function doesn't support item deletion",
            str(cm.exception)
        )

    def testObjectiveName(self):
        lp = LPX()
        with self.assertRaises(ValueError) as cm:
            lp.obj.name = 'a' * 256
        self.assertIn('name may be at most 255 chars', str(cm.exception))



"""Tests for proper memory deallocation of objects."""

import glpk
import weakref
import gc
import unittest


class GarbageCollectionTestCase(unittest.TestCase):
    """A garbage collection test case.

    In a nutshell, this test case provides an easy method to help
    identify potential memory leaks.

    This test case class wraps every test method with code that
    defines a list of weak references.  The test method may add items
    to this list with the reg registration method.  The idea is that
    registered items are those that the user expects to be freed.  The
    end of the wrapper code forces the garbage collector to collect,
    and then ensures that all the weak references added to the weak
    reference list are currently invalid."""
    def __init__(self, name=''):
        unittest.TestCase.__init__(self, name)
        oldfunc = getattr(self, name)

        class tuplesubclass(list):
            pass
        self.tuplesubclass = tuplesubclass

        # "Wrap" the test function function with this replacement.
        def replacement():
            self.weakrefs = []
            oldfunc()                      # Call the original callable obj.
            gc.collect()
            bad_objs = []
            for wr in self.weakrefs:
                ob = wr()
                if ob is not None:
                    bad_objs.append(ob)
            self.assertEqual(len(bad_objs), 0,  # Run final check.
                             'These objs not freed: %s' % bad_objs)
        replacement.__doc__ = oldfunc.__doc__
        setattr(self, name, replacement)

    def reg(self, ob):
        """The object registering function.

        This appends a weak reference to the object."""
        self.weakrefs.append(weakref.ref(ob))
        return ob

    def testLPCreation(self):
        """Test simple creation of a linear program."""
        reg = self.reg
        reg(glpk.LPX())

    def testLPDefinition(self):
        """Test creation of many objects."""
        reg = self.reg
        lp = reg(glpk.LPX())
        lp.name = 'some name'
        reg(lp.rows).add(2)
        reg(lp.cols).add(5)
        reg(lp.rows[0]).matrix = [1, 2, 3, 4, 5]
        reg(lp.rows[1]).name = 'second row'
        for c in reg(iter(reg(lp.cols))):
            reg(c).bounds = 1, 2
        reg(reg(lp.cols)[0]).name
        reg(lp.obj)[:] = [6, 7, 8, 9, 10]
        sum([v for v in reg(iter(reg(lp.obj)))])



import glpk


def solve_sat(expression):
    # trivial case
    if len(expression) == 0:
        return []

    # otherwise count vars
    numvars = max([max([abs(v) for v in clause]) for clause in expression])

    # empty LP instance
    lp = glpk.LPX()

    # stop annoying messages
    glpk.env.term_on = False

    # as many columns as there are literals
    lp.cols.add(2*numvars)
    # literal must be between false and true
    for col in lp.cols:
        col.bounds = 0.0, 1.0

    # function to compute column index
    def lit2col(lit):
        return [2*(-lit)-1, 2*lit-2][lit > 0]

    # ensure "oppositeness" of literals
    for i in xrange(1, numvars+1):
        lp.rows.add(1)
        lp.rows[-1].matrix = [(lit2col(i), 1.0), (lit2col(-i), 1.0)]
        # must sum to exactly 1
        lp.rows[-1].bounds = 1.0

    # ensure "trueness" of each clause
    for clause in expression:
        lp.rows.add(1)
        lp.rows[-1].matrix = [(lit2col(lit), 1.0) for lit in clause]
        # at least one literal must be true
        lp.rows[-1].bounds = 1, None

    # try to solve the relaxed problem
    retval = lp.simplex()

    # should not fail in this fashion
    assert retval is None
    # ff no relaxed solution, no exact sol
    if lp.status != 'opt':
        return None

    for col in lp.cols:
        col.kind = int

    # try to solve this integer problem
    retval = lp.integer()
    # should not fail in this fashion
    assert retval is None
    if lp.status != 'opt':
        return None

    return [col.value > 0.99 for col in lp.cols[::2]]

exp = [
    (-1, -3, -4),
    (2, 3, -4),
    (1, -2, 4),
    (1, 3, 4),
    (-1, 2, -3)
]
print(solve_sat(exp))



import glpk

def maxflow(capgraph, s, t):
    node2rnum = {}                      # Map non-source/sink nodes to row num.
    for nfrom, nto, cap in capgraph:
        if nfrom!=s and nfrom!=t:
            node2rnum.setdefault(nfrom, len(node2rnum))
        if nto!=s and nto!=t:
            node2rnum.setdefault(nto, len(node2rnum))

    lp = glpk.LPX()                     # Empty LP instance.
    glpk.env.term_on = False            # Stop annoying messages.
    lp.cols.add(len(capgraph))          # As many columns cap-graph edges.
    lp.rows.add(len(node2rnum))         # As many rows as non-source/sink nodes

    for row in lp.rows: row.bounds = 0  # Net flow for non-source/sink is 0.

    mat = []                            # Will hold constraint matrix entries.

    for colnum, (nfrom, nto, cap) in enumerate(capgraph):
        lp.cols[colnum].bounds = 0, cap # Flow along edge bounded by capacity.

        if nfrom == s:
            lp.obj[colnum] =  1.0       # Flow from source increases flow value
        elif nto == s:
            lp.obj[colnum] = -1.0       # Flow to source decreases flow value

        if nfrom in node2rnum:          # Flow from node decreases its net flow
            mat.append((node2rnum[nfrom], colnum, -1.0))
        if nto in node2rnum:            # Flow to node increases its net flow
            mat.append((node2rnum[nto], colnum, 1.0))

    lp.obj.maximize = True              # Want source s max flow maximized.
    lp.matrix = mat                     # Assign 0 net-flow constraint matrix.

    lp.simplex()                        # This should work unless capgraph bad.

    return [(nfrom, nto, col.value)     # Return edges with assigned flow.
            for col, (nfrom, nto, cap) in zip(lp.cols, capgraph)]

capgraph = [ ('s','o',3), ('s','p',3), ('o','p',2), ('o','q',3),
             ('p','r',2), ('q','r',4), ('q','t',2), ('r','t',3) ]
print(maxflow(capgraph, 's', 't'))

capgraph = [('s','a',4), ('s','b',1), ('a','b',2.5), ('a','t',1), ('b','t',4)]
print(maxflow(capgraph, 's', 't'))



"""Tests for operations upon LP rows and columns."""


from glpk import LPX, env
import unittest


class AddVectorTestCase:
    """Collection of tests for adding rows or columns."""
    def testVectorIsInitiallyEmpty(self):
        """Test that vector collection is initially empty."""
        self.assertEqual(len(self.vc), 0)

    def testAddVector(self):
        """Tests adding a single vector."""
        self.vc.add(1)
        self.assertEqual(len(self.vc), 1)

    def testFirstVectorIndexedAtZero(self):
        """Tests the first vector added has index 0."""
        vec_index = self.vc.add(1)
        self.assertEqual(vec_index, 0)

    def testAddMultipleVectors(self):
        """Tests adding multiple vectors."""
        vec_index = self.vc.add(6)
        self.assertEqual(vec_index, 0)
        self.assertEqual(len(self.vc), 6)

    def testMultipleAddMultipleVectors(self):
        """Tests multiple additions of multiple vectors."""
        num_to_add, running_sum = [1, 5, 3, 3, 3], 0
        for add_now in num_to_add:
            vec_index = self.vc.add(add_now)
            self.assertEqual(vec_index, running_sum)
            running_sum += add_now
            self.assertEqual(len(self.vc), running_sum)

    def testNonpositiveAdd(self):
        """Tests for errors when adding non-postive number of vectors."""
        self.assertRaises(ValueError, self.vc.add, 0)
        self.assertRaises(ValueError, self.vc.add, -1)
        self.assertRaises(ValueError, self.vc.add, -100)

    def testBadAddType(self):
        """Tests for errors when adding non-int number of vectors."""
        self.assertRaises(TypeError, self.vc.add, (2,))
        self.assertRaises(TypeError, self.vc.add, [5, 4])
        self.assertRaises(TypeError, self.vc.add, 'foobar')


class IndexVectorTestCase:
    """Collection of tests for indexing/accessing rows and columns."""
    def testEmptySingleIntegerLookup(self):
        """Tests integer indexing of an empty vector collection."""
        for i in (0, -4, 5):
            def f(): self.vc[i]
            self.assertRaises(IndexError, f)

    @unittest.skipIf(env.version < (4, 7), 'TODO')
    def testEmptySingleStringLookup(self):
        """Tests string indexing of an empty vector collection."""

        def f(key):
            self.vc[key]
        self.assertRaises(KeyError, f, "foo")

    def testEmptyTupleIntegerLookup(self):
        """Tests integer-tuple indexing of an empty vector collection."""
        def f(): self.vc[0, 2]
        self.assertRaises(IndexError, f)

    @unittest.skipIf(env.version < (4, 7), 'TODO')
    def testEmptyTupleStringLookup(self):
        """Tests string-tuple indexing of an empty vector collection."""

        def f():
            self.vc["foo", "bar"]
        self.assertRaises(KeyError, f)

    def testEmptyTupleMixedLookup(self):
        """Tests mixed-tuple indexing of an empty vector collection."""
        def f(): self.vc[2, "foo"]
        self.assertRaises(LookupError, f)

    def testEmptySliceLookup(self):
        """Tests slice lookups on empty vector collections."""
        self.assertEqual(len(self.vc[4:9]), 0)
        self.assertEqual(len(self.vc[:-1]), 0)
        self.assertEqual(len(self.vc[-3:]), 0)

    def testIntegerIndexingOfSingleVector(self):
        """Tests indexing a single vector with an int."""
        self.vc.add(1)
        vector = self.vc[0]
        self.assertEqual(vector.index, 0)

    def testIntegerIndexingOfMultipleVector(self):
        """Tests indexing many vectors with ints."""
        self.vc.add(6)
        v0, v2, v5 = self.vc[0], self.vc[2], self.vc[5]
        self.assertEqual(v0.index, 0)
        self.assertEqual(v2.index, 2)
        self.assertEqual(v5.index, 5)

    def testNegativeIntegerIndexingOfMultipleVectors(self):
        """Tests indexing vectors with negative ints."""
        self.vc.add(9)
        v8, v2, v4 = self.vc[-1], self.vc[-7], self.vc[-5]
        self.assertEqual(v8.index, 8)
        self.assertEqual(v2.index, 2)
        self.assertEqual(v4.index, 4)

    def testIntegerTupleIndexingOfMultipleVector(self):
        """Tests indexing many vectors with int tuples."""
        self.vc.add(6)
        self.assertEqual([v.index for v in self.vc[0, 5, 2]], [0, 5, 2])

    def testNegativeIntegerTupleIndexingOfMultipleVectors(self):
        """Tests indexing vectors with negative int tuples."""
        self.vc.add(9)
        self.assertEqual([v.index for v in self.vc[-1, -7, -5]], [8, 2, 4])

    def testAnyIntegerTupleIndexingOfMultipleVectors(self):
        """Tests indexing vectors with positive/negative int tuples."""
        self.vc.add(11)
        self.assertEqual([v.index for v in self.vc[-1, 7, -4]], [10, 7, 7])

    @unittest.skipIf(env.version < (4, 7), 'TODO')
    def testStringIndexingOfSingleVector(self):
        """Tests indexing a single vector with a string."""
        self.vc.add(5)
        self.vc[2].name = 'foo'
        self.assertEqual(self.vc['foo'].index, 2)
        self.assertEqual(self.vc['foo'].name, 'foo')

    @unittest.skipIf(env.version < (4, 7), 'TODO')
    def testStringMembershipCheck(self):
        """Tests the 'in' operator for string indices."""
        self.vc.add(5)
        self.assertFalse('foo' in self.vc)
        self.vc[2].name = 'foo'
        self.assertTrue('foo' in self.vc)
        self.assertFalse('bar' in self.vc)

    @unittest.skipIf(env.version < (4, 7), 'TODO')
    def testStringTupleIndexingOfMultipleVector(self):
        """Tests indexing multiple vectors with a string tuple."""
        self.vc.add(5)
        self.vc[2].name = 'foo'
        self.vc[4].name = 'bar'
        self.vc[0].name = 'blammo'
        self.assertEqual([v.index for v in self.vc['foo', 'blammo', 'bar']],
                         [2, 0, 4])
        self.assertEqual([v.name for v in self.vc['bar', 'foo', 'blammo']],
                         ['bar', 'foo', 'blammo'])

    @unittest.skipIf(env.version < (4, 7), 'TODO')
    def testStringNameReset(self):
        """Test that a name is no longer a valid index once set/reset."""
        self.vc.add(5)
        self.vc[2].name = 'foo'
        self.assertEqual(self.vc['foo'].name, 'foo')

        def f(old, new):
            self.vc[old].name = new
        f('foo', 'bar')  # First, set the name to something new.
        self.assertRaises(KeyError, f, 'foo', 'bar')  # Lookup on old name.
        del self.vc['bar'].name
        self.assertRaises(KeyError, f, 'bar', 'foo')  # Lookup on deleted name.

    def testSliceIndexingOfVectors(self):
        """Tests indexing vectors by slice."""
        to_add = 37
        i = list(range(to_add))
        self.vc.add(to_add)
        self.assertEqual([v.index for v in self.vc[:5]], i[:5])
        self.assertEqual([v.index for v in self.vc[:-4]], i[:-4])
        self.assertEqual([v.index for v in self.vc[-4:]], i[-4:])
        self.assertEqual([v.index for v in self.vc[::-1]], i[::-1])
        self.assertEqual([v.index for v in self.vc[4:29]], i[4:29])
        self.assertEqual([v.index for v in self.vc[4:2]], i[4:2])
        self.assertEqual([v.index for v in self.vc[4:30:3]], i[4:30:3])

    def testBadTypeIndexingOfVectors(self):
        """Tests indexing with non-integers/strings gives type errors."""
        self.vc.add(1)

        def f(): self.vc[{1: 2, 3: 4}]
        self.assertRaises(TypeError, f)

    def testBadTypeMultipleIndexingOfVectors(self):
        """Tests bad type indices mixed with good indices."""
        self.vc.add(5)

        def f():
            self.vc[-1, (1, 2, 3, 4), 2]
        self.assertRaises(TypeError, f)

    def testIteratorAccessToVectors(self):
        """Tests that the vector collection may be iterated upon."""
        to_add = 78
        self.vc.add(to_add)
        self.assertEqual([v.index for v in self.vc], list(range(to_add)))

    def testIteratorWriteAccessToVectors(self):
        """Tests that the vectors from iteration are mutable."""
        to_add = 12
        self.vc.add(to_add)
        for vec in self.vc:
            vec.name = 'x%d' % vec.index
        for vec in self.vc:
            self.assertEqual(vec.name, 'x%d' % vec.index)
        self.assertEqual(self.vc[3].name, 'x3')
        if env.version >= (4, 7):
            self.assertEqual(self.vc['x10'].index, 10)


class DeleteVectorTestCase:
    """Tests deleting vectors from the vector collection."""
    def moreSetUp(self):
        self.num_vecs = 5
        self.vc.add(self.num_vecs)
        for v in self.vc:  # Name them.
            v.name = 'x%d' % v.index

    @unittest.skipIf(env.version < (4, 7), 'TODO')
    def testDeleteOneVectorDecreasesLength(self):
        """Tests length of the vector collection after deleting one."""
        del self.vc[3]
        self.assertEqual(len(self.vc), self.num_vecs-1)
        self.assertEqual(self.vc[3].name, 'x4')
        self.assertTrue('x3' not in self.vc)

    def testDeleteTupleVectorDecreasesLength(self):
        """Tests deleting columns indexed by tuple."""
        del self.vc[4, -4, 0]
        self.assertEqual(len(self.vc), self.num_vecs-3)
        self.assertEqual(self.vc[0].name, 'x2')
        self.assertEqual(self.vc[1].name, 'x3')

    def testDeleteSimpleSliceVectorDecreasesLength(self):
        """Tests deleting columns indexed by slice."""
        del self.vc[:2]
        self.assertEqual(len(self.vc), self.num_vecs-2)
        self.assertEqual([v.name for v in self.vc], ['x2', 'x3', 'x4'])
        if env.version >= (4, 7):
            self.assertTrue('x0' not in self.vc)
            self.assertTrue('x1' not in self.vc)
        del self.vc[:]
        self.assertEqual(len(self.vc), 0)

    def testDeleteExtendedSliceVectorDecreasesLength(self):
        """Tests deleting columns indexed by an extended slice."""
        del self.vc[::2]
        self.assertEqual(len(self.vc), self.num_vecs // 2)
        self.assertEqual([v.name for v in self.vc],
                         ['x%d' % i for i in range(1, self.num_vecs, 2)])
        if env.version >= (4, 7):
            for i in range(0, self.num_vecs, 2):
                self.assertTrue(('x%d' % i) not in self.vc)

    def testDeleteVectorInvalidation(self):
        """Tests Bar invalidation once its index becomes out of bounds."""
        self.thebar = self.vc[-1]
        self.thebar.name  # This should have no problems.
        del self.vc[::2]
        with self.assertRaises(RuntimeError):
            self.thebar.name
        with self.assertRaises(RuntimeError):
            self.thebar.name = 5
        with self.assertRaises(RuntimeError):
            self.thebar.matrix
        del self.thebar


class BoundVectorTestCase:
    """Tests the setting of bounds.

    Note that this merely tests that once set, the bounds on the
    vectors are stored properly, not that the bounds are respected
    when it comes time to do the actual optimization."""
    def moreSetUp(self):
        self.vec = self.vc[self.vc.add(1)]

    def testInitiallyNoBounds(self):
        """Test that initial bounds are what we expect."""
        # The GLPK default (at least in version 4.14) is that row
        # variables are initially unbounded, and column variables are
        # constrained to equal 0.
        if self.vec.isrow:
            self.assertEqual(self.vec.bounds, (None, None))
        else:
            self.assertEqual(self.vec.bounds, (0.0, 0.0))

    def testSetUnbounded(self):
        """Tests setting a variable as unbounded."""
        self.vec.bounds = None
        self.assertEqual(self.vec.bounds, (None, None))

    def testSetEqualityBounds(self):
        """Tests setting equality bounds."""
        for num in [3, -2, 3.145159, 1234567.89]:
            self.vec.bounds = num
            self.assertEqual(self.vec.bounds, (num, num))

    def testSetLowerBounds(self):
        """Tests setting a lower bound."""
        for num in [3, -2, 3.145159, 1234567.89]:
            self.vec.bounds = num, None
            self.assertEqual(self.vec.bounds, (num, None))

    def testSetUpperBounds(self):
        """Tests setting an upper bound."""
        for num in [3, -2, 3.14159, 1234567.89]:
            self.vec.bounds = None, num
            self.assertEqual(self.vec.bounds, (None, num))

    def testDeleteBounds(self):
        """Tests setting a vector as unbounded through deletion."""
        self.vec.bounds = 3
        self.assertEqual(self.vec.bounds, (3, 3))
        del self.vec.bounds
        self.assertEqual(self.vec.bounds, (None, None))

    def testSetRangeBounds(self):
        """Tests setting a range bound."""
        bounds = [(2, 4), (-1, 0), (3.14159, 3.5), (3.3, 3.3), (-100, 300)]
        for bound in bounds:
            self.vec.bounds = bound
            self.assertEqual(self.vec.bounds, bound)

    def testManyBounds(self):
        """Tests setting many different types of bounds."""
        bounds = [(2, 4), None, (-1, 0), (4.5, None), 5, 123.4,
                  (3.14159, 3.5), (3.3, 3.3), None, (-100, 300),
                  -8, (1, None), (None, 7)]
        for bound in bounds:
            self.vec.bounds = bound
            if type(bound) == tuple:
                self.assertEqual(self.vec.bounds, bound)
            else:
                self.assertEqual(self.vec.bounds, (bound, bound))

    def testEmptyBounds(self):
        """Tests lower bounds greater than upper bounds is an error."""
        with self.assertRaises(ValueError):
            self.vec.bounds = (5, 4)

    def testBadTupleNumberBounds(self):
        """Tests that non-2-element-tuples are unacceptable bounds."""
        with self.assertRaises(TypeError):
            self.vec.bounds = (4,)
        with self.assertRaises(TypeError):
            self.vec.bounds = 2, 4, 5
        with self.assertRaises(TypeError):
            self.vec.bounds = None, 2, 3, 3

    def testBadTypeBounds(self):
        """Tests that bad types assigned as bounds are caught."""
        with self.assertRaises(TypeError):
            self.vec.bounds = "Hi!"
        with self.assertRaises(TypeError):
            self.vec.bounds = {1: 2}

    def testGoodBoundsKeptAfterBadSetAttempts(self):
        """Tests that bad bound sets do not overwrite old bounds."""
        self.vec.bounds = 3, 5
        try:
            self.vec.bounds = 4, 6, None
        except TypeError:
            pass
        self.assertEqual(self.vec.bounds, (3, 5))
        try:
            self.vec.bounds = 6, 4
        except ValueError:
            pass
        self.assertEqual(self.vec.bounds, (3, 5))


class ScaleVectorTestCase:
    """Tests setting row and column scaling."""
    def moreSetUp(self):
        self.num_vecs = 5
        self.vc.add(self.num_vecs)

    def testSetScaling(self):
        """Simple test to see that scaling factors are set."""
        self.vc[0].scale = 2
        self.vc[1].scale = 3.14159
        self.assertAlmostEqual(self.vc[0].scale, 2)
        self.assertAlmostEqual(self.vc[1].scale, 3.14159)

    def testNonpositiveScaling(self):
        """Test whether non-positive scale factors throw exceptions."""
        with self.assertRaises(ValueError):
            self.vc[0].scale = -1.0
        with self.assertRaises(ValueError):
            self.vc[1].scale = 0.0

    def testBadScaleType(self):
        """Tests whether non-numeric scale factors throw exceptions."""
        self.vc[0].scale = 3.14159
        with self.assertRaises(TypeError):
            self.vc[0].scale = complex(2, 2)
        with self.assertRaises(TypeError):
            self.vc[1].scale = None
        with self.assertRaises(TypeError):
            self.vc[2].scale = {}
        with self.assertRaises(TypeError):
            self.vc[3].scale = "foo"
        self.assertAlmostEqual(self.vc[0].scale, 3.14159)

    def testDeleteScale(self):
        """Testss whether deleting a scale factor throws exceptions."""
        with self.assertRaises(AttributeError):
            del self.vc[0].scale


class RowTestCase(unittest.TestCase):
    def moreSetUp(self): pass

    def setUp(self):
        self.lp = LPX()
        self.vc = self.lp.rows  # Vector collection is the row list.
        self.moreSetUp()


class ColumnTestCase(unittest.TestCase):
    def moreSetUp(self): pass

    def setUp(self):
        self.lp = LPX()
        self.vc = self.lp.cols  # Vector collection is the column list.
        self.moreSetUp()


class AddRowTestCase(AddVectorTestCase, RowTestCase):
    pass


class AddColumnTestCase(AddVectorTestCase, ColumnTestCase):
    pass


class IndexRowTestCase(IndexVectorTestCase, RowTestCase):
    pass


class IndexColumnTestCase(IndexVectorTestCase, ColumnTestCase):
    pass


class DeleteRowTestCase(DeleteVectorTestCase, RowTestCase):
    pass


class DeleteColumnTestCase(DeleteVectorTestCase, ColumnTestCase):
    pass


class BoundRowTestCase(BoundVectorTestCase, RowTestCase):
    pass


class BoundColumnTestCase(BoundVectorTestCase, ColumnTestCase):
    pass


class ScaleRowTestCase(ScaleVectorTestCase, RowTestCase):
    pass


class ScaleColumnTestCase(ScaleVectorTestCase, ColumnTestCase):
    pass


class ComparisonTestCase(unittest.TestCase):
    """Test the rich comparison operators on vectors."""
    def setUp(self):
        self.lp = LPX()
        self.r = self.lp.rows
        self.c = self.lp.cols
        self.r.add(5)
        self.c.add(4)

    def testEquality(self):
        """Simple tests of equality of the same vectors."""
        self.assertEqual(self.r[2], self.r[2])
        self.assertEqual(self.c[-1], self.c[3])

    def testInequality(self):
        """Simple tests of inequality of different vectors."""
        self.assertNotEqual(self.r[2], self.r[3])
        self.assertNotEqual(self.c[0], self.c[2])

    def testComparisonSameCollection(self):
        """Test that vectors in the same collection are ordered by index."""
        self.assertTrue(self.r[0] < self.r[1])
        self.assertTrue(self.r[2] > self.r[1])
        self.assertTrue(self.c[2] > self.c[1])
        self.assertTrue(self.c[2] < self.c[-1])

    def testComparisonDifferentCollection(self):
        """Test vector comparions in different collections."""
        a, b = self.r, self.c
        if b < a:
            a, b = b, a
        # A should be the lower bar collection.
        self.assertTrue(a[0] < b[0])
        self.assertTrue(b[1] > a[3])
        self.assertTrue(a[1] < b[0])
